
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Chris Walters
%%% August 2020
%%% This program fits the mixed logit
%%% model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% SET UP MATLAB %%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       %%Basic setup -- clear and launch parallel pool if available, add path for functions;
       clear all;
       try;
           parpool;
       end;
       addpath('functions')

       %%Set optimization options;
    	options_fmin=struct('Display','iter','MaxFunEvals',100000,'MaxIter',1000,'GradObj','off','LargeScale','off','UseParallel',true,'TolFun',1e-6);

        %%Set model parameters;

            %General parameters;
            rand_seed=10141985;
            rng(rand_seed);
            data_set='nprs';
            R=1000;
            R_fit=100000;
            R_post=10000;

            %Logit model specification
		%(set model to 'constant' for no beta heterogeneity, 'types_restrict' for discrete types with probability unrelated to alpha,
		%'types' for discrete types with probability depending on alpha.
            model='types_restrict';
            covs=1;
            covs_interact=0;
            savename=[data_set '_' model '_covs' num2str(covs)  '.mat'];

       %%Switches;
       estimate_logit=0;
       fit_logit=0;
       posteriors=0;
       posteriors_sim=0;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% IMPORT DATA %%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        %%Pull in data;
        data_raw=csvread([data_set '.csv']);
        l=1;
        id=data_raw(:,l); l=l+1;
        cluster=data_raw(:,l); l=l+1;
        Y=data_raw(:,l); l=l+1;
        D=data_raw(:,l); l=l+1;
        if isequal(data_set,'nprs')==1;
            industry=data_raw(:,l); l=l+1;
        end;
        X=data_raw(:,l:end);
        J=length(unique(id));
        N=length(Y);
        X=X - mean(X,1);
        X_D=zeros(N,0);
        if covs_interact==1;
            X=X(:,1);
            X_D=X;
        end;
        if covs~=1;
            X=zeros(N,0);
            X_D=zeros(N,0);
        end;
        ncovs=length(X(1,:));
        data=[id Y D X X_D];

         %%Construct job-level objects;
         N_B=dummyvar(id)'*D;
         N_W=dummyvar(id)'*(1-D);
         C_B=dummyvar(id)'*(Y.*D);
         C_W=dummyvar(id)'*(Y.*(1-D));
         data_job=[unique(id) N_B N_W C_B C_W];

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% ESTIMATE LOGIT MODEL (TABLE 5) %%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if estimate_logit==1;

        %%%Starting values;
        nparams = 3 + length(X(1,:)) + length(X_D(1,:)) ...
                    + (isequal(model,'norm_restrict')+isequal(model,'lognorm_restrict')+isequal(model,'types_restrict')) ...
                    + 2*(isequal(model,'norm')+isequal(model,'lognorm')+isequal(model,'types')+isequal(model,'chisq_restrict')) ...
                    + 3*isequal(model,'chisq');
        theta_0=randn(nparams,1);

        %%%Sim draws;
        sims=zeros(J,R,2);
        halt=net(haltonset(2,'Skip',10),J*R);
        sims(:,:,1)=norminv(reshape(halt(:,1),[R J])');
        sims(:,:,2)=norminv(reshape(halt(:,2),[R J])');

        %%%Compute estimates;
        tic
        theta_hat=fminunc(@(x)likelihood(x,data,sims,model,ncovs,0),theta_0,options_fmin);
        toc

        %%%%Process results;

            %Standard errors;
             [theta_hat Q_hat exitflag output G H]=fminunc(@(x)likelihood(x,data,sims,model,ncovs,0),theta_hat,options_fmin);
             [x,~,~,~,~,~,G] = lsqnonlin(@(x)likelihood(x,data,sims,model,ncovs,1),theta_hat,[],[],optimset('MaxIter',0,'TolFun',1000000000));
             meat=zeros(length(theta_hat),length(theta_hat));
             for j=1:length(unique(id));
                g=G(j,:)';
                meat=meat+(g*g');
             end;
             V=inv(H)*meat*inv(H);
             SE=sqrt(diag(V));
             obj=-Q_hat;

           %%Display results (Table 5);
           [theta_0 theta_hat SE]
	   obj
	   length(theta_hat)
           length(unique(id))

            %%Save;
            results={theta_hat,SE,obj,H,V,model,covs,theta_0};
            save(savename,'results');

    end;

 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%%%%%% LOGIT MODEL FIT (EXPORT DATA FOR FIG A1) %%%%%%%%%%%%
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 if fit_logit==1;

     %Load results;
     load(savename);
     l=1;
     theta_hat=results{l}; l=l+1;
     SE=results{l}; l=l+1;
     obj=results{l}; l=l+1;
     H=results{l}; l=l+1;
     V=results{l}; l=l+1;
     model=results{l}; l=l+1;
     covs=results{l}; l=l+1;
     theta_0=results{l};

     %Unpack params;
        continuous=isequal(model,'norm_restrict') + isequal(model,'lognorm_restrict') + isequal(model,'chisq_restrict') ...
                      + isequal(model,'norm') + isequal(model,'lognorm') + isequal(model,'chisq');
        restrict=isequal(model,'norm_restrict') + isequal(model,'lognorm_restrict') + isequal(model,'chisq_restrict') ...
                      + isequal(model,'types_restrict');
        types=isequal(model,'types') + isequal(model,'types_restrict');
        chisq=isequal(model,'chisq') + isequal(model,'chisq_restrict');
        lognorm=isequal(model,'lognorm') + isequal(model,'lognorm_restrict');
        theta=theta_hat;
        l=1;
        alpha_0=theta(l); l=l+1;
        sigma_alpha=exp(theta(l)); l=l+1;
        beta_0=theta(l); l=l+1;
        sigma_beta=0; rho=0; beta_2=0;
        if continuous==1;
            sigma_beta=exp(theta(l)); l=l+1;
            if restrict==0;
                rho=(exp(2*theta(l))-1)/(exp(2*theta(l))+1); l=l+1;
            end;
            if chisq==1;
                beta_2=theta(l); l=l+1;
            end;
        end;
        if types==1;
            delta_0=theta(l); l=l+1;
            delta_alpha=0;
            if restrict==0;
                delta_alpha=theta(l); l=l+1;
            end;
        end;
        gamma=theta(l:(l+ncovs-1)); l=l+ncovs;
        gamma_D=theta(l:end);
        Xgamma=X*gamma;

     %%%Export data;
        N_B=dummyvar(id)*N_B;
        N_W=dummyvar(id)*N_W;
        data_out=[id Y D Xgamma N_B N_W];
        csvwrite('fit_data',data_out);
        stop


 end;


 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%%%%%% FORM POSTERIORS %%%%%%%%%%%%%
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 if posteriors==1;

     %Load results;
     load(savename);
     l=1;
     theta_hat=results{l}; l=l+1;
     SE=results{l}; l=l+1;
     obj=results{l}; l=l+1;
     H=results{l}; l=l+1;
     V=results{l}; l=l+1;
     model=results{l}; l=l+1;
     covs=results{l}; l=l+1;
     theta_0=results{l};

     %%%Compute posteriors;
     sims_post=zeros(J,R_post,2);
     halt=net(haltonset(2,'Skip',10),J*R_post);
     sims_post(:,:,1)=norminv(reshape(halt(:,1),[R_post J])');
     sims_post(:,:,2)=norminv(reshape(halt(:,2),[R_post J])');
     p=get_posteriors(theta_hat,data,sims_post,model);

     %%%Summarize posteriors;
     suspect=(N_B==2).*(N_W==2).*(C_B==0).*(C_W==2);
     if isequal(data_set,'ray')==1;
        suspect=(N_B==4).*(N_W==4).*(C_B==0).*(C_W==4);
     end;

     %%%List unique posteriors;
     post_results=[N_W N_B C_W C_B p];

     min(p)
     mean(p)
     prctile(p,90)
     max(p)
     min(p(suspect==1))
     mean(p(suspect==1))
     prctile(p(suspect==1),90)
     max(p(suspect==1))
     stp


 end;

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %%%%%%% POSTERIOR SIMULATIONS %%%%%%%%%%%%%
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   if posteriors_sim==1;


                              %Load results;
                              load(savename);
                              l=1;
                              theta_hat=results{l}; l=l+1;
                              SE=results{l}; l=l+1;
                              obj=results{l}; l=l+1;
                              H=results{l}; l=l+1;
                              V=results{l}; l=l+1;
                              model=results{l}; l=l+1;
                              covs=results{l}; l=l+1;
                              theta_0=results{l};

                              %Define parameters;
                              num_jobs=20000;
                              design=[3 1];
                              X=[-0.25 0 0.25];
                              output=[];

                              %Loop over designs;
                              for i=1:length(X);

                                  %Extract covs;
                                  x=X(i);

                                  %Simulate data;
                                  [coefs datasim]=data_sim(theta_hat,model,x,num_jobs,design);

                                  %Compute realized discrimination;
                                  alpha=coefs(:,1);
                                  beta=coefs(:,2);
                                  p_w=1./(1+exp(-(alpha+x)));
                                  p_b=1./(1+exp(-(alpha-beta+x)));
                                  disc=(p_b./p_w)<=0.8;

                                  %Compute posteriors;
                                  sims_post=zeros(num_jobs,R_post,2);
                                  halt=net(haltonset(2,'Skip',10),num_jobs*R_post);
                                  sims_post(:,:,1)=norminv(reshape(halt(:,1),[R_post num_jobs])');
                                  sims_post(:,:,2)=norminv(reshape(halt(:,2),[R_post num_jobs])');
                                  results_sim=get_posteriors_sim(theta_hat,datasim,sims_post,model);
                                  [combos ida combo]=unique(results_sim(:,1:4),'rows');
                                  for t=1:max(combo);
                                      output=[output; mean(results_sim(combo==t,:),1) mean(combo==t) x];
                                  end;

                              end;


                              %Save results;
                              csvwrite(['posteriors_' data_set '_' model '.csv'],output);
                              stop
      end;
