% Ben Scuderi
% Last Updated: 9/16/2020
% Implement empirical calculation of minimax and bayes decision rules.

clear all

% PART 0) SETUP: insheet Nunley freqeuncies & posteriors and grid, set cost ratio & rules to test
freqs = readtable( '../data/logit_frequencies_withX_2_2.csv'); % moment constraints
rules = readtable( '../data/logit_frequencies_withX_5_5.csv'); % evidence configurations
grid  = readtable('../grids/logit_grid_QLmono_withX_K55.csv'); % 4D discretized support grid for G
range = [1 1000];                                              % test rules in this range (rank of evidence of threshold callback configuration)

% PART 1) Analysis for "80% rule"

ratio = 0.8;  % fix ratio, implies 80% posterior threshold decision rule under bayes risk

% a. estimate bayes risk of decision rules
bayes_risk(rules,range,'logit2',ratio);
bayes     = readtable(strcat('../results/bayes_risk_withX_'  ,'logit2','_loss_ratio',num2str(10*ratio),'.csv'));
[~,min_b] = min(bayes.risk)

% b. estimate minimax risk of decision rules
lp_minimax_risk(freqs,grid,rules,range,'logit2',ratio);
minmx     = readtable(strcat('../results/minimax_risk_withX_','logit2','_loss_ratio',num2str(10*ratio),'.csv'));
[~,min_m] = min(minmx.risk)

% c. output minimax G, and report some quantities of interest
lp_minimax_EB(freqs,grid,rules,min_m,'logit2',ratio)
lp_minimax_G( freqs,grid,rules,min_m,'logit2',ratio);

% d. Plot minimax and bayes risk -- figure 6
fig = figure
plot([minmx.post(min_m) minmx.post(min_m)],[-1 30],'LineWidth', 1)
hold on;
plot([0.80000           0.80000          ],[-1 30],'LineWidth', 1,'Color',uint8([180 180 180]))
hold on;
ax1 = gca;
hold on;
plot(minmx.post, bayes.risk, 'LineWidth', 2,'Color',uint8([180 180 180]))
hold on;
ax1.ColorOrderIndex = 1;
plot(minmx.post, minmx.risk, 'LineWidth', 2)
hold on;
ylim([-1 30])
xlabel('Posterior Cutoff')
ylabel('Risk x 100')
legend('Maximum Risk','Bayes Risk','Location','southoutside','Orientation','horizontal')
ax2 = axes('Position',[.45 .45 .43 .43])
box  on;
plot(minmx.post,bayes.risk,'-','LineWidth', 2,'Color',uint8([180 180 180]))
hold on;
ax2.ColorOrderIndex = 1;
plot(minmx.post,minmx.risk,'-','LineWidth', 2)
hold on;
plot(minmx.post(min_m),minmx.risk(min_m),'r*')
plot(0.8,              bayes.risk(min_b),'r*')
ylim([-0.075 0])
xlim([ 0.74  1])
print(fig,strcat('../results/risk_profile_','logit2','_loss_ratio',num2str(10*ratio)),'-dpng', '-r1000');


% PART 2) Compare rules with different ratio parameters (kappa)

% a. Loop over parameter values
parfor ratio = 1:9
    lp_minimax_risk(freqs,grid,rules,range,'logit2',ratio/10);
end

% b. Write results to csv
minmx_compare = [];
for ratio = 1:9
    minmx = readtable(strcat('../results/minimax_risk_withX_','logit2','_loss_ratio',num2str(ratio),'.csv'));
    minmx = removevars(minmx,{'rule','a_w_h','a_w_l','a_b_h','a_b_l','c_w_h','c_w_l','c_b_h','c_b_l','freq','e_logit'});
    [~,m] = min(minmx.risk);
    minmx_compare(ratio,:) = [ratio/10 table2array(minmx(m,:))];
end
outsheet(minmx_compare,'ratio,post_thresh,risk,pr_accuse,pr_innocent,pr_acquit_innocent,pr_accuse_guilty','../results/minimax_vs_bayes_cutoffs.csv')

% c. Plot results
fig2 = figure
plot((0:0.1:1)',[0; minmx_compare(:,2); 1],'-o','LineWidth', 1.25)
hold on;
plot((0:0.1:1)',(0:0.1:1)'                ,'--','LineWidth', 1.25,'Color',uint8([180 180 180]))
xlabel('Bayes threshold')
ylabel('Minimax threshold')
xlim([-0.05 1.05])
ylim([-0.05 1.05])
print(fig2,'../results/minimax_vs_bayes_cutoffs','-dpng', '-r1000');
