% Ben Scuderi
% Last Updated: 9/16/20
% Program to estimate moments of callback distribution using MM and QP,
% and estimate posterior bounds on the fraction of nondiscriminators
% using LP. Additionally, compute CNS bootstrap distribution and the numerical
% BS standard errors on moment estimates.
% Inputs: lists of counts of callbacks by design and configuration


% I. PRELIMINARIES

% Setup - try parallell pool, add path for functions
clear all
delete(gcp('nocreate'))
try;
    parpool;
end;
addpath('functions')

% a. Switches: run only if switch == 1
% BM           ; AGCV           ; NUNLEY         ;
b_gmm       = 0; a_gmm       = 0; n_gmm       = 0; % Standard GMM
b_cons      = 0; a_cons      = 0; n_cons      = 0; % Shape-Constrained QP-GMM
b_mono      = 0; a_mono      = 0; n_mono      = 0; % Shape-Constrained QP-GMM + Monotonicity
b_nbs_cons  = 0; a_nbs_cons  = 0; n_nbs_cons  = 0; % HONG & LI
b_nbs_mono  = 0; a_nbs_mono  = 0; n_nbs_mono  = 0; % HONG & LI (Monotonicity)
b_cns_cons  = 0; a_cns_cons  = 0; n_cns_cons  = 0; % CNS
b_cns_mono  = 0; a_cns_mono  = 0; n_cns_mono  = 0; % CNS (Monotonicity)
b_cns_revm  = 0; a_cns_revm  = 0; n_cns_revm  = 0; % CNS (Reverse Monotonicity)
b_cns_trce  = 0; a_cns_trce  = 0; n_cns_trce  = 0; % CNS (No Discrimination)

% b. Insheet data
bm          = csvread('../data/bm_counts.csv',    1,0);
agcv        = csvread('../data/agcv_counts.csv',  1,0);
nunley      = csvread('../data/nunley_counts.csv',1,0);
nunley      = nunley(nunley(:,1)>0&nunley(:,2)>0,:); % Drop 0,4 and 4,0 designs;

% c. Insheet grid options
grid_K1     = csvread('../grids/logit_G_quadL_150.csv', 1,0);
grid_K2     = csvread('../grids/logit_G_quadL_900.csv', 1,0);
p_w_K1      = grid_K1(:,1);
p_b_K1      = grid_K1(:,2);
p_w_K2      = grid_K2(:,1);
p_b_K2      = grid_K2(:,2);

% e. Set bootstrap parameters
reps        = 1000;
seed        = 23145;

% f. Define headers (variable names) for saving results
header_J    = 'J_cmd,J_omd';
header_G    = 'P_w,P_b,M_cmd,M_omd,Centered_omd';
header_b    = 'N_w,N_b,C_w,C_b,f_raw,f_gmm_cmd,f_gmm_omd';
header_d    = 'm,k';
header_k    = 'key_moments';
header_gbsC = 'P_w,P_b,Moment,SE_BS';
header_gbsK = 'Moment,SE_BS';
header_ests = 'N_w,N_b,C_w,C_b,f_raw,f_qp_cmd,f_qp_omd';
header_j_st = 'J_cmd,J_omd';
header_bnds = 'N_w,N_b,Strata,Strata_Prob,Bound_Tr,Bound_LT,Bound_UT';
header_bd_s = 'N_w,N_b,C_w,C_b,f_omd,Bound_Tr,Bound_LT,Bound_UT';
header_dist = 'P_w,P_b,G_cmd,G_omd';
header_cns  = 'J_omd';
header_nbsC = 'P_w,P_b,Moment,SE_NumBS,5pctile_NumBS,95pctile_NumBS';
header_nbsK = 'Moment,SE_NumBS,5pctile_NumBS,95pctile_NumBS';


% II. STANDARD GMM ESTIMATION (via QP)

% a. BERTRAND-MULLAINATHAN
if b_gmm == 1
    [J_b d_b G_b b_b se_b] = gmm_ests(bm               );
    [C_b key_b]            = recenter(G_b,         'bm');
    [C_bs_b key_bs_b]      = gmm_bs(  bm,reps,seed,'bm');

    outsheet(J_b,       header_J,    '../results/bm_gmm_J.csv');
    outsheet([G_b C_b], header_G,    '../results/bm_gmm_M.csv');
    outsheet(b_b,       header_b,    '../results/bm_gmm_ests.csv');
    outsheet(d_b,       header_d,    '../results/bm_gmm_df.csv');
    outsheet(key_b,     header_k,    '../results/bm_gmm_key_moments.csv');
    outsheet(C_bs_b,    header_gbsC, '../results/bm_gmm_bs_centered.csv');
    outsheet(key_bs_b,  header_gbsK, '../results/bm_gmm_bs_key_moments.csv');
end

% b. AG-CV
if a_gmm == 1
    [J_a d_a G_a b_a se_a] = gmm_ests(agcv                 );
    [C_a key_a]            = recenter(G_a,           'agcv');
    [C_bs_a key_bs_a]      = gmm_bs(  agcv,reps,seed,'agcv');

    outsheet(J_a,       header_J,    '../results/agcv_gmm_J.csv');
    outsheet([G_a C_a], header_G,    '../results/agcv_gmm_M.csv');
    outsheet(b_a,       header_b,    '../results/agcv_gmm_ests.csv');
    outsheet(d_a,       header_d,    '../results/agcv_gmm_df.csv');
    outsheet(key_a,     header_k,    '../results/agcv_gmm_key_moments.csv');
    outsheet(C_bs_a,    header_gbsC, '../results/agcv_gmm_bs_centered.csv');
    outsheet(key_bs_a,  header_gbsK, '../results/agcv_gmm_bs_key_moments.csv');
end

% c. NUNLEY
if n_gmm == 1
    [J_n d_n G_n b_n se_n] = gmm_ests(nunley                   );
    [C_n key_n]            = recenter(G_n,             'nunley');
    [C_bs_n key_bs_n]      = gmm_bs(  nunley,reps,seed,'nunley');

    outsheet(J_n,       header_J,    '../results/nunley_gmm_J.csv');
    outsheet([G_n C_n], header_G,    '../results/nunley_gmm_M.csv');
    outsheet(b_n,       header_b,    '../results/nunley_gmm_ests.csv');
    outsheet(d_n,       header_d,    '../results/nunley_gmm_df.csv');
    outsheet(key_n,     header_k,    '../results/nunley_gmm_key_moments.csv');
    outsheet(C_bs_n,    header_gbsC, '../results/nunley_gmm_bs_centered.csv');
    outsheet(key_bs_n,  header_gbsK, '../results/nunley_gmm_bs_key_moments.csv');
end


% III. SHAPE-CONSTRAINED QP ESTIMATION

% a. BERTRAND-MULLAINATHAN
if b_cons == 1
    [ests_b J_b G_b] = qp_ests(      bm,             p_w_K1,p_b_K1,0,ones(size(p_b_K1)));
    [bounds_b]       = lp_bounds(    bm, ests_b(:,7),p_w_K2,p_b_K2,  ones(size(p_b_K2)));
    [bounds_b_s]     = lp_bounds_sep(bm, ests_b(:,7),p_w_K2,p_b_K2);
    [~,~,M_b,~,~]    = gmm_ests([ests_b(:,1:4) ests_b(:,7)]);
    [C_b key_b]      = recenter(M_b,'bm');

    outsheet(ests_b,    header_ests, '../results/bm_qp_ests.csv');
    outsheet(J_b,       header_j_st, '../results/bm_qp_J.csv');
    outsheet(G_b,       header_dist, '../results/bm_qp_dists.csv');
    outsheet([M_b C_b], header_G   , '../results/bm_qp_moments.csv');
    outsheet(key_b,     header_k   , '../results/bm_qp_key_moments.csv');
    outsheet(bounds_b,  header_bnds, '../results/bm_qp_bounds.csv');
    outsheet(bounds_b_s,header_bd_s, '../results/bm_qp_bounds_sep.csv');
end

% b. AG-CV
if a_cons == 1
    [ests_a J_a G_a] = qp_ests(      agcv,             p_w_K1,p_b_K1,0,ones(size(p_b_K1)));
    [bounds_a]       = lp_bounds(    agcv, ests_a(:,7),p_w_K2,p_b_K2,  ones(size(p_b_K2)));
    [bounds_a_s]     = lp_bounds_sep(agcv, ests_a(:,7),p_w_K2,p_b_K2);
    [~,~,M_a,~,~]    = gmm_ests([ests_a(:,1:4) ests_a(:,7)]);
    [C_a key_a]      = recenter(M_a,'agcv');

    outsheet(ests_a,    header_ests, '../results/agcv_qp_ests.csv');
    outsheet(J_a,       header_j_st, '../results/agcv_qp_J.csv');
    outsheet(G_a,       header_dist, '../results/agcv_qp_dists.csv');
    outsheet([M_a C_a], header_G   , '../results/agcv_qp_moments.csv');
    outsheet(key_a,     header_k   , '../results/agcv_qp_key_moments.csv');
    outsheet(bounds_a,  header_bnds, '../results/agcv_qp_bounds.csv');
    outsheet(bounds_a_s,header_bd_s, '../results/agcv_qp_bounds_sep.csv');

end

% c. NUNLEY
if n_cons == 1
    [ests_n J_n G_n] = qp_ests(      nunley,             p_w_K1,p_b_K1,0,ones(size(p_b_K1)));
    [bounds_n]       = lp_bounds(    nunley, ests_n(:,7),p_w_K2,p_b_K2,  ones(size(p_b_K2)));
    [bounds_n_s]     = lp_bounds_sep(nunley, ests_n(:,7),p_w_K2,p_b_K2);
    [~,~,M_n,~,~]    = gmm_ests([ests_n(:,1:4) ests_n(:,7)]);
    [C_n key_n]      = recenter(M_n,'nunley');

    outsheet(ests_n,    header_ests, '../results/nunley_qp_ests.csv');
    outsheet(J_n,       header_j_st, '../results/nunley_qp_J.csv');
    outsheet(G_n,       header_dist, '../results/nunley_qp_dists.csv');
    outsheet([M_n C_n], header_G   , '../results/nunley_qp_moments.csv');
    outsheet(key_n,     header_k   , '../results/nunley_qp_key_moments.csv');
    outsheet(bounds_n,  header_bnds, '../results/nunley_qp_bounds.csv');
    outsheet(bounds_n_s,header_bd_s, '../results/nunley_qp_bounds_sep.csv');
end


% IV. SHAPE-CONSTRAINED QP ESTIMATION + MONOTONICITY

% a. BERTRAND-MULLAINATHAN
if b_mono == 1
    [ests_b J_b G_b] = qp_ests(  bm,     p_w_K1,p_b_K1,0,(p_w_K1 >= p_b_K1));
    [~,~,M_b,~,~]    = gmm_ests(        [ests_b(:,1:4) ests_b(:,7)]);
    [C_b key_b]      = recenter(         M_b,'bm'    );
    [bounds_b]       = lp_bounds(bm,     ests_b(:,7),p_w_K2,p_b_K2,(p_w_K2 >= p_b_K2));

    outsheet(ests_b,    header_ests, '../results/bm_qp_ests_mono.csv');
    outsheet(J_b,       header_j_st, '../results/bm_qp_J_mono.csv');
    outsheet(G_b,       header_dist, '../results/bm_qp_dists_mono.csv');
    outsheet([M_b C_b], header_G   , '../results/bm_qp_moments_mono.csv');
    outsheet(key_b,     header_k   , '../results/bm_qp_key_moments_mono.csv');
    outsheet(bounds_b,  header_bnds, '../results/bm_qp_bounds_mono.csv');
end

% b. AG-CV
if a_mono == 1
    [ests_a J_a G_a] = qp_ests(  agcv,   p_w_K1,p_b_K1,0,(p_w_K1 >= p_b_K1));
    [~,~,M_a,~,~]    = gmm_ests(        [ests_a(:,1:4) ests_a(:,7)]);
    [C_a key_a]      = recenter(         M_a,'agcv'  );
    [bounds_a]       = lp_bounds(agcv,   ests_a(:,7),p_w_K2,p_b_K2,(p_w_K2 >= p_b_K2));

    outsheet(ests_a,    header_ests, '../results/agcv_qp_ests_mono.csv');
    outsheet(J_a,       header_j_st, '../results/agcv_qp_J_mono.csv');
    outsheet(G_a,       header_dist, '../results/agcv_qp_dists_mono.csv');
    outsheet([M_a C_a], header_G   , '../results/agcv_qp_moments_mono.csv');
    outsheet(key_a,     header_k   , '../results/agcv_qp_key_moments_mono.csv');
    outsheet(bounds_a,  header_bnds, '../results/agcv_qp_bounds_mono.csv');
end

% c. NUNLEY
if n_mono == 1
    [ests_n J_n G_n] = qp_ests(  nunley, p_w_K1,p_b_K1,0,(p_w_K1 >= p_b_K1));
    [~,~,M_n,~,~]    = gmm_ests(        [ests_n(:,1:4) ests_n(:,7)])
    [C_n key_n]      = recenter(         M_n,'nunley');
    [bounds_n]       = lp_bounds(nunley, ests_n(:,7),p_w_K2,p_b_K2,ones(size(p_b_K2))); %(p_w_K2 >= p_b_K2)); XX

    outsheet(ests_n,    header_ests, '../results/nunley_qp_ests_mono.csv');
    outsheet(J_n,       header_j_st, '../results/nunley_qp_J_mono.csv');
    outsheet(G_n,       header_dist, '../results/nunley_qp_dists_mono.csv');
    outsheet([M_n C_n], header_G   , '../results/nunley_qp_moments_mono.csv');
    outsheet(key_n,     header_k   , '../results/nunley_qp_key_moments_mono.csv');
    outsheet(bounds_n,  header_bnds, '../results/nunley_qp_bounds_mono.csv');
end


% V. HONG & LI NUMERICAL BOOTSTRAP PROCEDURE

% a. BERTRAND-MULLAINATHAN
if b_nbs_cons == 1
    [centered key J_bs_centered J_bs_key] = qp_num_bs(bm,    p_w_K1,p_b_K1,reps,seed,ones(size(p_b_K1)),'bm'    );
    outsheet(centered,      header_nbsC , '../results/bm_qp_centered_numbs.csv');
    outsheet(key,           header_nbsK , '../results/bm_qp_key_moments_numbs.csv');
    outsheet(J_bs_centered, []          , '../results/bm_qp_numbs_centered_reps.csv');
    outsheet(J_bs_key,      []          , '../results/bm_qp_numbs_key_reps.csv');
end

% b. AG-CV
if a_nbs_cons == 1
    [centered key J_bs_centered J_bs_key] = qp_num_bs(agcv,  p_w_K1,p_b_K1,reps,seed,ones(size(p_b_K1)),'agcv'  );
    outsheet(centered,      header_nbsC , '../results/agcv_qp_centered_numbs.csv');
    outsheet(key,           header_nbsK , '../results/agcv_qp_key_moments_numbs.csv');
    outsheet(J_bs_centered, []          , '../results/agcv_qp_numbs_centered_reps.csv');
    outsheet(J_bs_key,      []          , '../results/agcv_qp_numbs_key_reps.csv');
end

% c. NUNLEY
if n_nbs_cons == 1
    [centered key J_bs_centered J_bs_key] = qp_num_bs(nunley,p_w_K1,p_b_K1,reps,seed,ones(size(p_b_K1)),'nunley');
    outsheet(centered,      header_nbsC , '../results/nunley_qp_centered_numbs.csv');
    outsheet(key,           header_nbsK , '../results/nunley_qp_key_moments_numbs.csv');
    outsheet(J_bs_centered, []          , '../results/nunley_qp_numbs_centered_reps.csv');
    outsheet(J_bs_key,      []          , '../results/nunley_qp_numbs_key_reps.csv');
end


% VI. HONG & LI NUMERICAL BOOTSTRAP PROCEDURE + MONOTONICITY

% a. BERTRAND-MULLAINATHAN
if b_nbs_mono == 1
    [centered key J_bs_centered J_bs_key] = qp_num_bs(bm,    p_w_K1,p_b_K1,reps,seed,(p_w_K1 >= p_b_K1),'bm'    );
    outsheet(centered,      header_nbsC , '../results/bm_qp_centered_numbs_mono.csv');
    outsheet(key,           header_nbsK , '../results/bm_qp_key_moments_numbs_mono.csv');
    outsheet(J_bs_centered, []          , '../results/bm_qp_numbs_centered_reps_mono.csv');
    outsheet(J_bs_key,      []          , '../results/bm_qp_numbs_key_reps_mono.csv');
end

% b. AG-CV
if a_nbs_mono == 1
    [centered key J_bs_centered J_bs_key] = qp_num_bs(agcv,  p_w_K1,p_b_K1,reps,seed,(p_w_K1 >= p_b_K1),'agcv'  );
    outsheet(centered,      header_nbsC , '../results/agcv_qp_centered_numbs_mono.csv');
    outsheet(key,           header_nbsK , '../results/agcv_qp_key_moments_numbs_mono.csv');
    outsheet(J_bs_centered, []          , '../results/agcv_qp_numbs_centered_reps_mono.csv');
    outsheet(J_bs_key,      []          , '../results/agcv_qp_numbs_key_reps_mono.csv');
end

% c. NUNLEY
if n_nbs_mono == 1
    [centered key J_bs_centered J_bs_key] = qp_num_bs(nunley,p_w_K1,p_b_K1,reps,seed,(p_w_K1 >= p_b_K1),'nunley');
    outsheet(centered,      header_nbsC , '../results/nunley_qp_centered_numbs_mono.csv');
    outsheet(key,           header_nbsK , '../results/nunley_qp_key_moments_numbs_mono.csv');
    outsheet(J_bs_centered, []          , '../results/nunley_qp_numbs_centered_reps_mono.csv');
    outsheet(J_bs_key,      []          , '../results/nunley_qp_numbs_key_reps_mono.csv');
end


% VII. CNS BOOTSTRAP PROCEDURE

% a. BERTRAND-MULLAINATHAN
if b_cns_cons == 1
    [J_b J_bs_b p_bs_b bs_reps_b] = qp_cns(bm,    p_w_K1,p_b_K1,reps,seed,ones(size(p_b_K1)));
    outsheet([J_b;p_bs_b],  header_cns  , '../results/bm_qp_cns_J.csv');
    outsheet(J_bs_b,        header_cns  , '../results/bm_qp_cns_dists.csv');
    outsheet(bs_reps_b,     []          , '../results/bm_qp_cns_reps.csv');
end

% b. AG-CV
if a_cns_cons == 1
    [J_a J_bs_a p_bs_a bs_reps_a] = qp_cns(agcv,  p_w_K1,p_b_K1,reps,seed,ones(size(p_b_K1)));
    outsheet([J_a;p_bs_a],  header_cns  , '../results/agcv_qp_cns_J.csv');
    outsheet(J_bs_a,        header_cns  , '../results/agcv_qp_cns_dists.csv');
    outsheet(bs_reps_a,     []          , '../results/agcv_qp_cns_reps.csv');
end

% c. NUNLEY
if n_cns_cons == 1
    [J_n J_bs_n p_bs_n bs_reps_n] = qp_cns(nunley,p_w_K1,p_b_K1,reps,seed,ones(size(p_b_K1)));
    outsheet([J_n;p_bs_n],  header_cns  , '../results/nunley_qp_cns_J.csv');
    outsheet(J_bs_n,        header_cns  , '../results/nunley_qp_cns_dists.csv');
    outsheet(bs_reps_n,     []          , '../results/nunley_qp_cns_reps.csv');
end


% VIII. CNS BOOTSTRAP PROCEDURE + MONOTONICITY

% a. BERTRAND-MULLAINATHAN
if b_cns_mono == 1
    [J_b J_bs_b p_bs_b bs_reps_b] = qp_cns(bm,    p_w_K1,p_b_K1,reps,seed,(p_w_K1 >= p_b_K1));
    outsheet([J_b;p_bs_b],  header_cns  , '../results/bm_qp_cns_J_mono.csv');
    outsheet(J_bs_b,        header_cns  , '../results/bm_qp_cns_dists_mono.csv');
    outsheet(bs_reps_b,     []          , '../results/bm_qp_cns_reps_mono.csv');
end

% b. AG-CV
if a_cns_mono == 1
    [J_a J_bs_a p_bs_a bs_reps_a] = qp_cns(agcv,  p_w_K1,p_b_K1,reps,seed,(p_w_K1 >= p_b_K1));
    outsheet([J_a;p_bs_a],  header_cns  , '../results/agcv_qp_cns_J_mono.csv');
    outsheet(J_bs_a,        header_cns  , '../results/agcv_qp_cns_dists_mono.csv');
    outsheet(bs_reps_a,     []          , '../results/agcv_qp_cns_reps_mono.csv');
end

% c. NUNLEY
if n_cns_mono == 1
    [J_n J_bs_n p_bs_n bs_reps_n] = qp_cns(nunley,p_w_K1,p_b_K1,reps,seed,(p_w_K1 >= p_b_K1));
    outsheet([J_n;p_bs_n],  header_cns  , '../results/nunley_qp_cns_J_mono.csv');
    outsheet(J_bs_n,        header_cns  , '../results/nunley_qp_cns_dists_mono.csv');
    outsheet(bs_reps_n,     []          , '../results/nunley_qp_cns_reps_mono.csv');
end


% IX. CNS BOOTSTRAP PROCEDURE + REVERSE MONOTONICITY

% a. BERTRAND-MULLAINATHAN
if b_cns_revm == 1
    [J_b J_bs_b p_bs_b bs_reps_b] = qp_cns(bm,    p_w_K1,p_b_K1,reps,seed,(p_w_K1 <= p_b_K1));
    outsheet([J_b;p_bs_b],  header_cns  , '../results/bm_qp_cns_J_revm.csv');
    outsheet(J_bs_b,        header_cns  , '../results/bm_qp_cns_dists_revm.csv');
    outsheet(bs_reps_b,     []          , '../results/bm_qp_cns_reps_revm.csv');
end

% b. AG-CV
if a_cns_revm == 1
    [J_a J_bs_a p_bs_a bs_reps_a] = qp_cns(agcv,  p_w_K1,p_b_K1,reps,seed,(p_w_K1 <= p_b_K1));
    outsheet([J_a;p_bs_a],  header_cns  , '../results/agcv_qp_cns_J_revm.csv');
    outsheet(J_bs_a,        header_cns  , '../results/agcv_qp_cns_dists_revm.csv');
    outsheet(bs_reps_a,     []          , '../results/agcv_qp_cns_reps_revm.csv');
end

% c. NUNLEY
if n_cns_revm == 1
    [J_n J_bs_n p_bs_n bs_reps_n] = qp_cns(nunley,p_w_K1,p_b_K1,reps,seed,(p_w_K1 <= p_b_K1));
    outsheet([J_n;p_bs_n],  header_cns  , '../results/nunley_qp_cns_J_revm.csv');
    outsheet(J_bs_n,        header_cns  , '../results/nunley_qp_cns_dists_revm.csv');
    outsheet(bs_reps_n,     []          , '../results/nunley_qp_cns_reps_revm.csv');
end


% X. CNS BOOTSTRAP PROCEDURE + TRACE

% a. BERTRAND-MULLAINATHAN
if b_cns_trce == 1
    [J_b J_bs_b p_bs_b bs_reps_b] = qp_cns(bm,    p_w_K1,p_b_K1,reps,seed,(p_w_K1 == p_b_K1));
    outsheet([J_b;p_bs_b],  header_cns  , '../results/bm_qp_cns_J_trace.csv');
    outsheet(J_bs_b,        header_cns  , '../results/bm_qp_cns_dists_trace.csv');
    outsheet(bs_reps_b,     []          , '../results/bm_qp_cns_reps_trace.csv');
end

% b. AG-CV
if a_cns_trce == 1
    [J_a J_bs_a p_bs_a bs_reps_a] = qp_cns(agcv,  p_w_K1,p_b_K1,reps,seed,(p_w_K1 == p_b_K1));
    outsheet([J_a;p_bs_a],  header_cns  , '../results/agcv_qp_cns_J_trace.csv');
    outsheet(J_bs_a,        header_cns  , '../results/agcv_qp_cns_dists_trace.csv');
    outsheet(bs_reps_a,     []          , '../results/agcv_qp_cns_reps_trace.csv');
end

% c. NUNLEY
if n_cns_trce == 1
    [J_n J_bs_n p_bs_n bs_reps_n] = qp_cns(nunley,p_w_K1,p_b_K1,reps,seed,(p_w_K1 == p_b_K1));
    outsheet([J_n;p_bs_n],  header_cns  , '../results/nunley_qp_cns_J_trace.csv');
    outsheet(J_bs_n,        header_cns  , '../results/nunley_qp_cns_dists_trace.csv');
    outsheet(bs_reps_n,     []          , '../results/nunley_qp_cns_reps_trace.csv');
end

delete(pool);
