********************************
*** Chris Walters
*** August 2020
*** This program replicates estimates
*** from "Reasonable Doubt"
********************************

	********************************
	**** SET UP STATA **************
	********************************
	
		***Basic setup
		clear all
		cap set trace off
		cap log close
		cap set more off
		set matsize 10000
		set maxvar 30000
		
		***Set directories 
		local root ""
		local data "`root'/data"
		local results "`root'/results"
		
		***Switches to select tasks
		local setup=0
		local descriptives=0
		local dependence=0
		local posterior_bounds=0
		local logit_posteriors=0
		local det_curve=0
		local logit_fit=0
		local bayes_minimax=0
		local moments=0
		local logit_frequencies=0
		local grids=0

				
	**************************************************
	**** SET UP DATA SETS ****************************
	**************************************************	
	if `setup'==1 {
	
		****Set up BM data ***********************	
		
			*Load data file
			use "`data'/lakisha_aer.dta", clear

			*Define variables and select sample
			drop id
			egen id=group(adid)
			gen Y=call
			gen B=race=="b"
			bys id: gen N=_N
			bys id: egen N_B=sum(B)
			keep if N==4 & N_B==2
			
			*Save cleaned data file
			save "`data'/bm.dta", replace		
			
			*Outsheet collapsed dataset of counts
			use "`data'/bm.dta", clear
			g N_w = 1 - B
			g N_b = B
			g C_w = N_w*Y
			g C_b = N_b*Y
			collapse (sum) N_w N_b C_w C_b, by(id)
			g F = 1
			collapse (sum) F, by(N_w N_b C_w C_b)
			outsheet using "`data'/bm_counts.csv", comma names replace 
			
		****Set up NPRS data ***********************
		
			*Load data file
			use "`data'/bejeap_race_discrim_walters_stata13.dta", clear

			*Define variables and select sample
			egen id=group(job_id)
			gen B=black
			gen Y=call_back
			gen business=acc_major +econ_major +fin_major +mkt_major +mgt_major
			bys id: gen N=_N
			keep if N==4
			gen id2=id
			drop id
			egen id=group(id2)
			drop id2
			egen industry=group(ind_*)	
				
			*Save stata data
			save "`data'/nprs.dta", replace

			*Save matlab data
			keep id Y B industry underemp female high_ses no_gap u6_current u12_current u3_current u3_past u6_past business intern gpa
			gen cluster=id
			order id cluster Y B industry underemp female high_ses no_gap u6_current u12_current u3_current u3_past u6_past business intern gpa
			sort id B Y
			outsheet using "`data'/nprs.csv", nonames comma replace
			
			*Outsheet collapsed dataset of counts
			use "`data'/nprs.dta", clear
			g N_w = 1 - B
			g N_b = B
			g C_w = N_w*Y
			g C_b = N_b*Y
			collapse (sum) N_w N_b C_w C_b, by(job_id)
			g F = 1
			collapse (sum) F, by(N_w N_b C_w C_b)
			g tmp = C_w + C_b == 0
			gsort -tmp N_w N_b C_w C_b
			drop tmp
			outsheet using "`data'/nunley_counts.csv", comma names replace 
		
		****Set up AGCV data ***********************	
			
			*Load raw data
			use "`data'/agcv_rawdata.dta", clear
		
			*Format data and select sample
			egen id=group(id_offer)
			gen B=1-sex
			gen Y=callback
			bys id: gen N=_N
			bys id: egen N_B = sum(B)
			gen N_W = N - N_B
			gen D_44 = (N_B==4) & (N_W==4)
			keep if (D_44==1)
			drop N N_B
			gen id2=id
			drop id
			egen id=group(id2)
			drop id2
			
			*Code resume order
			gen date_num=date(fecha_sent,"DMY")
			split hour_sent, parse(":") ge(t_)
			destring t_*, replace
			sort id date_num t_1 t_2
			bys id: gen order_cw=_n
				
			*Save cleaned data file
			save "`data'/agcv.dta", replace
			
			*Outsheet collapsed dataset of counts
			use "`data'/agcv.dta", clear
			g N_w = sex
			g N_b = 1 - sex
			g C_w = N_w*Y
			g C_b = N_b*Y
			collapse (sum) N_w N_b C_w C_b, by(id)
			g F = 1
			collapse (sum) F, by(N_w N_b C_w C_b)
			tsset C_w C_b  // Fill in missing cell in AGCV data
			tsfill, full
			replace N_w = 4 if mi(N_w)
			replace N_b = 4 if mi(N_b)
			replace F   = 0 if mi(F  ) 
			outsheet using "`data'/agcv_counts.csv", comma names replace 
		
	}
	
	**************************************************
	**** DESCRIPTIVE STATISTICS (TABLE 1) ************
	**************************************************	
	if `descriptives'==1 {
	
		****Loop over data sets
		foreach d in bm nprs agcv {
		
			**Load data set
			use "`data'/`d'.dta", clear
			
			**Callback rates
			sum Y
			bys B: sum Y
			
			**Treatment/control difference
			reg Y B, cluster(id)

		}

	}
	
	**************************************************
	**** DEPENDENCE CHECKS (TABLE 2) *****************
	**************************************************	
	if `dependence'==1 {
	
		
		***NPRS: Analysis using resume characteristics (cols 1-2)

			*Load data
			use "`data'/nprs.dta", clear
			
			*Run leave-out regressions
			gen gap = 1-no_gap
			local listvars ""
			foreach x of varlist black female high_ses gpa business gap u3_current u6_current u12_current u3_past u6_past u12_past {
			
				*Compute leave-out mean
				cap bys id: egen Xbar_`x'=mean(`x')
				replace Xbar_`x' = (4/3)*(Xbar_`x' - (`x'/4))
				
				*Run regression
				reg Y `x' Xbar_`x', cluster(id)
				
				*Unclustered regression to store estimates
				reg Y `x' Xbar_`x'
				estimates store `x'
				local listvars "`listvars' Xbar_`x'"
			}
			
			*Joint test
			suest black female high_ses gpa business gap u3_current u6_current u12_current u3_past u6_past u12_past , cluster(id)
			test `listvars'

		****AGCV: Resume order (last 3 rows of Panel C in cols 3-7)
		
			*Load data
			use "`data'/agcv.dta", clear
		
			*Code order dummies
			qui tab order_cw, ge(orderdum_)
			drop orderdum_8
			
			*Test for equal callback rates by order
			reg Y orderdum*, cluster(id)
			test orderdum_1=orderdum_2=orderdum_3=orderdum_4=orderdum_5=orderdum_6=orderdum_7=0
			local chi=7*r(F)
			disp `chi'
			
			*Regression of callback rate on order
			reg Y order_cw, cluster(id)
			
			*Regression of callback on prev fraction female
			sort id order_cw
			bys id: gen prev_female=sum(sex)
			replace prev_female = prev_female - sex
			gen frac_female=prev_female/4
			reg callback sex frac_female, cluster(id_offer)
			
			
		****AGCV: Resume order chi-squared tests (cols 3-7)

			
			* a) split data into two four-app spells per job
			use   "`data'/agcv.dta", clear
			keep    id Y order 
			ren    (id Y order) (job_id callback order )
			sort   job_id order 
			order  job_id order callback 

			g 	    spell = ceil(order/4) 
			replace order = order - (spell-1)*4
			reshape wide callback, i(job_id spell) j(order) 
			ren (callback*) (c_*) 
			egen num_c = rowtotal(c_1-c_4) 
			g 	 freq  = 1

			collapse (sum) freq, by(c_1-c_4 num_c) 

			tostring c_1   c_2   c_3   c_4, replace
			g seq  = c_1 + c_2 + c_3 + c_4
			drop 	 c_1   c_2   c_3   c_4	

			order num_c seq freq 
			sort  num_c seq

			preserve 
			collapse (sum) freq, by(num_c) 
			ren freq tot
			tempfile temp
			save   "`temp'" 
			restore 
			merge m:1 num_c using "`temp'", nogen
			outsheet using "'results'/agcv_4app_sequence_freq.csv", comma names replace 

			g p 		= comb(4,num_c)^(-1)
			g X_sq 		= (freq/tot - p)^2/(p*(1-p)/tot)
			g df 		= 1 
			collapse (sum) X_sq df freq, by(num_c) 
			replace df 	= df - 1
			g p_val 	= chi2tail(df,X_sq) 
			drop if mi(p_val) 
			ren  freq obs
			order num_c X_sq df p_val obs
			outsheet using "'results/agcv_4app_p_vals.csv'", comma names replace 
			
			
			* b) retain one eight-app spell
			use   "`data'/agcv.dta", clear
			keep    id Y order 
			ren    (id Y order) (job_id callback order )
			sort   job_id order 
			order  job_id order callback   

			preserve 
			clear 
			set obs 2
			g c_1 = _n - 1
			expand 2
			bys c_1: g c_2 = _n - 1
			expand 2
			bys c_1 c_2: g c_3 = _n - 1
			expand 2
			bys c_1 c_2 c_3: g c_4 = _n - 1
			expand 2
			bys c_1 c_2 c_3 c_4: g c_5 = _n - 1
			expand 2
			bys c_1 c_2 c_3 c_4 c_5: g c_6 = _n - 1
			expand 2
			bys c_1 c_2 c_3 c_4 c_5 c_6: g c_7 = _n - 1
			expand 2
			bys c_1 c_2 c_3 c_4 c_5 c_6 c_7: g c_8 = _n - 1

			g num_c =    c_1 +   c_2 +   c_3 +   c_4 +   c_5 +   c_6 +   c_7 +  c_8
			g avg   = (1*c_1 + 2*c_2 + 3*c_3 + 4*c_4 + 5*c_5 + 6*c_6 + 7*c_7 +8*c_8)/num_c
			tostring     c_1     c_2     c_3     c_4     c_5     c_6     c_7    c_8, replace
			g seq   =    c_1 +   c_2 +   c_3 +   c_4 +   c_5 +   c_6 +   c_7 +  c_8
			drop 	     c_1     c_2     c_3     c_4     c_5     c_6     c_7    c_8
			tempfile temp
			save "`temp'"  
			restore 

			reshape wide callback, i(job_id) j(order) 
			ren (callback*) (c_*) 

			g num_c = c_1 + c_2 + c_3 + c_4 + c_5 + c_6 + c_7 + c_8
			tostring  c_1   c_2   c_3   c_4   c_5   c_6   c_7   c_8, replace
			g seq   = c_1 + c_2 + c_3 + c_4 + c_5 + c_6 + c_7 + c_8
			drop 	  c_1   c_2   c_3   c_4   c_5   c_6   c_7   c_8
			g freq  = 1

			collapse (sum) freq, by(num_c seq)  
			merge 1:1 num_c seq using "`temp'", nogen 
			replace freq = 0 if mi(freq) 
			 
			order num_c seq freq
			sort  num_c seq 

			preserve 
			collapse (sum) freq, by(num_c) 
			ren freq tot
			tempfile temp
			save   "`temp'" 
			restore 
			merge m:1 num_c using "`temp'", nogen
			order num_c seq freq tot 
			outsheet using "'results'/agcv_8app_sequence_freq.csv", comma names replace 

			g p 			= comb(8,num_c)^(-1)
			g X_sq 			= (freq/tot - p)^2/(p*(1-p)/tot)
			g df 			= 1 
			collapse (sum) X_sq df freq, by(num_c) 
			g zeros 		= comb(8,num_c)-df
			g p 			= comb(8,num_c)^(-1)
			replace X_sq 	= X_sq + zeros*(p^2/(p*(1-p)/freq))
			replace df 	 	= df+zeros - 1
			g p_val 		= chi2tail(df,X_sq) 
			drop if mi(p_val) 
			drop zeros p 
			ren  freq obs
			order num_c X_sq df p_val obs
			outsheet using "'results'/agcv_8app_p_vals.csv", comma names replace 

		
	}
	
	**************************************************
	**** POSTERIOR BOUND PLOTS (FIGS 1-3) *************
	**************************************************	
	if `posterior_bounds'==1 {
	
		***Load data on posterior bounds
		
			*Load data set 
			import excel "`data'/bound_data.xlsx", sheet("raw_bounds") firstrow clear
			
			*Generate key variables
			egen c=group(c_b c_w a_b a_w)
			gen labvar=""
			foreach n of numlist 0/4 {
				foreach m of numlist 0/4 {
					replace labvar="(`n',`m')" if c_w==`n' & c_b==`m'
				}
			}
			gen post_eq=1-lp_posterior_bound_eq
			gen post_geq=1-lp_posterior_bound_geq
			gen post_leq=1-lp_posterior_bound_leq
			gsort - post_eq
	
		
		***BM (Fig 1)
		preserve
		keep if dataset=="bm"
		expand 2 if _n==_N
		replace post_eq=0.13 if _n==_N
		replace labvar="All" if _n==_N
		graph bar post_eq, over(labvar, sort(lp_posterior_bound_eq)) ///
			ytitle("Lower bound on posterior probability") ylabel(0(0.1)1,nogrid) graphregion(color(white))
		restore

		****NPRS (Fig 2)
		preserve
		keep if dataset=="nunley"
		replace post_eq=post_leq
		gsort - post_eq
		gen minus_post_eq=1-post_eq
		keep if _n<=10
		expand 2 if _n==_N
		replace post_eq=0.173 if _n==_N
		replace labvar="All" if _n==_N
		replace a_w=0 if _n==_N
		replace a_b=0 if _n==_N
		gen i=_n
		egen design=group(a_w a_b)
		separate post_eq, by(design)
		graph bar post_eq?, over(i, ///
				 relabel(1 "(3,0)" 2 "(2,0)" 3 "(2,1)" 4 "(1,2)" 5 "(1,0)" 6 "(2,0)" 7 "(1,1)" 8 "(2,1)" 9 "(1,0)" 10 "(1,1)" 11 "All") ///
				 sort(minus_post_eq)) nofill ///
				 legend(order(4 2 3 1) lab(1 "All designs") lab(2 "(1,3) design") lab(4 "(3,1) design") lab(3 "(2,2) design")) ///
				ytitle("Lower bound on posterior probability") ylabel(0(0.1)1,nogrid) graphregion(color(white))
		restore

		****AGCV (Fig 3)
		preserve
		keep if dataset=="agcv"
		gsort - post_eq
		keep if _n<=10
		expand 2 if _n==_N
		replace post_geq=0.064 if _n==_N
		replace post_leq=0.142 if _n==_N
		replace labvar="All" if _n==_N
		gen maxpost=1-post_geq
		replace maxpost=1-post_leq if post_leq>post_geq
			graph bar post_leq post_geq, over(labvar, sort(maxpost)) ///
				ytitle("Lower bound on posterior probability") ylabel(0(0.1)1,nogrid) graphregion(color(white)) ///
				legend(order(1 2) lab(1 "Pr( {it:p{sub:f}} > {it:p{sub:m}} | {it:C{sub:f}} , {it:C{sub:m}} )") lab(2 "Pr( {it:p{sub:m}} > {it:p{sub:f}} | {it:C{sub:f}} , {it:C{sub:m}} )"))
		restore				
			
		
	}
	
	**************************************************
	* LOGIT MODEL PREDICTIONS AND POSTERIORS (FIG 4) *
	**************************************************	
	if `logit_posteriors'==1 {

		******Define logit parameters from Table 5
		local alpha_0=-4.931
		local sigma_alpha=4.988
		local beta_0=4.046
		local tau_0=-1.586
		local tau_alpha=0
		
		*****Define design and simulation parameters
		set seed 10141985
		local J=50000000
		local A_w_min=1
		local A_w_max=3
		local A_b_min=1
		local A_b_max=3
		local A_min=4
		local A_max=4

		******Set up results storage
		matrix T=J(10000,13,.)
		local r=1
		
		******Simulate p's
		set obs `J'
		gen alpha=`alpha_0'+`sigma_alpha'*invnormal(uniform())
		gen p_disc=1/(1+exp(-(`tau_0'+`tau_alpha'*alpha)))
		gen D=uniform()<=p_disc
		gen beta=-D*`beta_0'
		gen p_w_h=1/(1+exp(-(alpha+0.5)))
		gen p_w_l=1/(1+exp(-(alpha-0.5)))
		gen p_b_h=1/(1+exp(-(alpha+beta+0.5)))
		gen p_b_l=1/(1+exp(-(alpha+beta-0.5)))
		sum p_disc
		local P_D=r(mean)
		gen pos_h = p_b_h - 0.8*p_w_h
		replace pos_h=0 if pos_h<=0
		gen pos_l = p_b_l - 0.8*p_w_l
		replace pos_l=0 if pos_l<=0
		gen pos = pos_h + pos_l 
		gen neg_h=0.8*p_w_h - p_b_h
		replace neg_h=0 if neg_h<=0
		gen neg_l=0.8*p_w_l - p_b_l
		replace neg_l=0 if neg_l<=0
		gen neg=neg_h+neg_l
		gen ratio=(p_w_l+p_w_h)/(p_b_l+p_b_h)
		gen logit_ratio=exp(ratio)/(1+exp(ratio))

		******Loop over possible designs
		foreach A_w of numlist `A_w_min'/`A_w_max' {
			foreach A_b of numlist `A_b_min'/`A_b_max' {
				if ((`A_w'+`A_b')<=`A_max') & ((`A_w'+`A_b')>=`A_min') {
					
					*****Loop over possible X configurations
					foreach A_w_h of numlist 0/`A_w' {
						foreach A_b_h of numlist 0/`A_b' {
							local A_w_l = `A_w'-`A_w_h'
							local A_b_l = `A_b'-`A_b_h'
							
							******Simulate callback outcomes in this design
							foreach g in w_h w_l b_h b_l {
								qui gen C_`g'=0
								if `A_`g''>0 {
									qui replace C_`g'=rbinomial(`A_`g'',p_`g')
								}
							}

							
							*****Loop over possible callback outcomes 
							foreach C_w_h of numlist 0/`A_w_h' {
								foreach C_w_l of numlist 0/`A_w_l' {
									foreach C_b_h of numlist 0/`A_b_h' {
										foreach C_b_l of numlist 0/`A_b_l' {
										
											********Calculate frequency
											qui gen f=1
											 foreach t in w_h w_l b_h b_l {
												gen f_`t'=1
												if `A_`t''>0 {
													qui replace f_`t'=comb(`A_`t'',`C_`t'')*(p_`t'^`C_`t'')*((1-p_`t')^(`A_`t''-`C_`t''))
												}
												qui replace f=f*f_`t'
											 }
											 

											 *****Fill in results
											 matrix T[`r',1]=`A_w_h'
											 matrix T[`r',2]=`A_w_l'
											 matrix T[`r',3]=`A_b_h'
											 matrix T[`r',4]=`A_b_l'
											 matrix T[`r',5]=`C_w_h'
											 matrix T[`r',6]=`C_w_l'
											 matrix T[`r',7]=`C_b_h'
											 matrix T[`r',8]=`C_b_l'
											 qui sum f
											 local f=r(mean)
											 matrix T[`r',9]=`f'
											 qui sum f if D==1
											 matrix T[`r',10]=r(mean)*(`P_D'/`f')
											 qui sum pos if C_w_h==`C_w_h' & C_w_l==`C_w_l' & C_b_h==`C_b_h' & C_b_l==`C_b_l'
											 matrix T[`r',11]=r(mean)
											 qui sum neg if C_w_h==`C_w_h' & C_w_l==`C_w_l' & C_b_h==`C_b_h' & C_b_l==`C_b_l'
											 matrix T[`r',12]=r(mean)
											 qui sum logit_ratio if C_w_h==`C_w_h' & C_w_l==`C_w_l' & C_b_h==`C_b_h' & C_b_l==`C_b_l'
											 matrix T[`r',13]=r(mean)
											 qui drop f*
											 local ++r
											 disp `r'
										}
									}
								}
							}
							
							***Drop simulated callbacks
							qui drop C*
							
						}
					}
				}
			}
		}
		
		
		*******Save results
		clear
		svmat T
		ren T1 A_w_h
		ren T2 A_w_l
		ren T3 A_b_h
		ren T4 A_b_l
		ren T5 C_w_h
		ren T6 C_w_l
		ren T7 C_b_h
		ren T8 C_b_l
		ren T9 f
		ren T10 post
		ren T11 E_pos
		ren T12 E_neg
		ren T13 E_logit
		drop if f==.
		outsheet using "`data'/logit_frequencies_withX_v2.csv", names comma replace

		*****Make graph of logit posteriors (Fig 4)
		insheet using "`data'/logit_frequencies_withX_v2.csv", names comma clear
		gen post_eq=post
		gsort - post_eq
		keep if ((a_w_h+a_w_l)==2) & ((a_b_h+a_b_l)==2)
		keep if ((a_w_h==2 & a_b_h==2) | (a_w_h==2 & a_b_h==0) | (a_w_h==0 & a_b_h==2) | (a_w_h==0 & a_b_h==0))
		gen c_w=c_w_h+c_w_l
		gen c_b=c_b_h+c_b_l
		egen c=group(c_w c_b)
		foreach w in 0 2 {
			foreach b in 0 2 {
				gen post_`w'`b'=post_eq if (a_w_h==`w' & a_b_h==`b')
				bys c: egen p_`w'`b'=mean(post_`w'`b')
			}
		}
		bys c: keep if _n==1
		gen i=_n
		expand 2
		bys i: gen j=_n
		drop if j==2 & i!=9
		replace i=10 if j==2 & i==9
		replace p_02=0.168 if i==10
		replace p_22=0.168 if i==10
		replace p_00=0.168 if i==10
		replace p_20=0.168 if i==10
		gen minus_post_eq=1-p_02
		replace minus_post_eq=1 if i==10
		graph bar p_02 p_22 p_00 p_20, over(i, ///
				 relabel(1 "(0,0)" 2 "(0,1)" 3 "(0,2)" 4 "(1,0)" 5 "(1,1)" 6 "(1,2)" 7 "(2,0)" 8 "(2,1)" 9 "(2,2)" 10 "All") ///
				 sort(minus_post_eq)) nofill ///
				 legend(lab(1 "LQ white, HQ black") lab(2 "HQ white, HQ black") ///
				 lab(3 "LQ white, LQ black") lab(4 "HQ white, LQ black")) ///
				ytitle("Posterior probability") ylabel(0(0.1)1,nogrid) graphregion(color(white))
			
		
	}
	
	**************************************************
	**** DET CURVE PLOT (FIG 5) **********************
	**************************************************	
	if `det_curve'==1 {
	
		/* Note: To execute this section it is necessary to generate simulated type 1 and type 2 error
			rates for a variety of possible application portfolios. These data sets are used below.
			to recreate these data files from scratch, run the part1/data/portfolios/portfolio_test.m
			matlab program. */
	
		*** Set directory
		clear
		cd "`data'/portfolios"
		
		***Import simulated investigation shares
		
			* Import HQ white/LQ black data
			save portfolio_data, replace emptyok
			local files : dir "results" files "portfolio_curve_500k_nochat_2covtypes_0ipairs_10depth_0pairs_*_*_0.5_1.csv"
			foreach file in `files' {
				insheet using "results/`file'", comma clear
				drop if _n > 1
				gen ipairs = 0
				gen max_depth = 10
				append using portfolio_data
				save portfolio_data, replace
			}
					
			* Import 5 pairs data
			local files : dir "results" files "portfolio_curve_500k_nochat_2covtypes_5ipairs_10depth_0pairs_*_*_0.5_1.csv"
			foreach file in `files' {
				insheet using "results/`file'", comma clear
				drop if _n > 1
				gen ipairs = 5
				gen max_depth = 10	
				append using portfolio_data
				save portfolio_data, replace
			}

			* Import 2 pairs data
			save portfolio_data, replace emptyok
			local files : dir "results" files "portfolio_curve_500k_nochat_2covtypes_2ipairs_4depth_0pairs_*_*_0.5_1.csv"
			foreach file in `files' {
				insheet using "results/`file'", comma clear
				drop if _n > 1
				gen ipairs = 2
				gen max_depth = 4
				append using portfolio_data
				save portfolio_data, replace
			}


		*Keep results corresponding to relevant region of graph, label 0.8 posterior
		keep if share_d_accused < 0.5
		keep if share_nond_acquit>0.988
		gen post = kappa/(gamma+kappa)
		gen diff = abs(post - 0.8)
		bys ipairs max_depth: egen mindiff=min(diff)
		gen f = diff==mindiff
		sort share_d_accused

		*Graph results, along with lines connecting dots
		graph twoway scatter share_nond_acquit share_d_accused if ipairs == 0 & max_depth == 10, mcolor(navy) msymbol(oh) mlwidth(vthin) connect(l) lcolor(navy) ///
			|| scatter share_nond_acquit share_d_accused if ipairs == 5 & max_depth == 10, mcolor(maroon) msymbol(oh) mlwidth(vthin) connect(l) lcolor(maroon)  ///
			|| scatter share_nond_acquit share_d_accused if ipairs == 2 & max_depth == 4, mcolor(forest_green) msymbol(oh) mlwidth(vthin) connect(l) lcolor(forest_green) ///
			|| function y=0.99, lcolor(gray) range(0 0.16) ///
			|| scatter share_nond_acquit share_d_accused if ipairs == 0 & max_depth == 10 & f==1, mcolor(navy) ///
			|| scatter share_nond_acquit share_d_accused if ipairs == 5 & max_depth == 10 & f==1, mcolor(maroon) ///
			|| scatter share_nond_acquit share_d_accused if ipairs == 2 & max_depth == 4 & f==1, mcolor(forest_green) ///
			|| function y = 1.022659-0.4407184*x, range(0.069316 0.0765) lcolor(forest_green) ///
			graphregion(color(white)) xtitle("Share of discriminators investigated") ytitle("Share of non-discriminators not investigated") ///
			legend(order(3 "2 pairs" 2 "5 pairs" 1 "5 pairs (HQ black, LQ white)")) ylabel(0.988(0.002)1, nogrid) xlabel(0(0.05)0.16)

		
	}
	
	
	**************************************************
	**** LOGIT MODEL FIT (FIG A1) **********************
	**************************************************	
	if `logit_fit'==1 {
	
		****Read in data from matlab
		insheet using "`data'/fit_data.csv", comma clear
		ren v1 id
		ren v2 Y
		ren v3 B
		ren v4 Xgamma
		ren v5 N_B
		ren v6 N_W
		
		****Define logit parameters from Table 5
		local alpha_0=-4.931
		local sigma_alpha=4.988
		local beta_0=4.046
		local tau_0=-1.586
		local tau_alpha=0
		
		****Define simulation parameters
		set seed 10141985
		local j=1
		local J=10000

		*****Calculate model-predicted outcomes for every job
		while `j'<=`J' {
		
			*Simulate outcomes
			qui gen alpha=`alpha_0'+`sigma_alpha'*invnormal(uniform())
			qui gen p_D=exp(`tau_0'+(`tau_alpha'*alpha))/(1+exp(`tau_0'+(`tau_alpha'*alpha)))
			qui gen Dsim=uniform()<p_D
			qui bys id: replace alpha=alpha[1]
			qui bys id: replace Dsim=Dsim[1]
			qui gen U=alpha-(`beta_0'*D*B)+Xgamma
			qui gen p_Y=1/(1+exp(-U))
			qui gen Ysim=uniform()<p_Y

			*Store results
			foreach c_w of numlist 0/4 {
				foreach c_b of numlist 0/4 {
					if (`c_w'+`c_b')<=4 {
						qui gen YB_sim=Ysim*B
						qui gen YW_sim=Ysim*(1-B)
						qui bys id: egen CB_sim=sum(YB_sim)
						qui bys id: egen CW_sim=sum(YW_sim)
						cap gen fsim_`c_w'_`c_b'=0
						qui replace fsim_`c_w'_`c_b' = fsim_`c_w'_`c_b' + (1/`J')*(CB_sim==`c_b')*(CW_sim==`c_w')
						qui drop YB_sim YW_sim CB_sim CW_sim
					}
				}
			}
			
			qui drop alpha p_D Dsim U p_Y Ysim
			local prog=`j'/`J'
			disp "`prog'"
			local ++j
		}
		preserve
		keep id fsim_*
		duplicates drop
		save "`data'/model_predictions.dta", replace
		restore

		****Calculate empirical f's and chi-sq goodness of fit stat
		matrix T=J(30,4,.)
		local t=1
		local C=0
		local df=0
		gen YB=Y*B
		gen YW=Y*(1-B)
		bys id: egen C_B=sum(YB)
		bys id: egen C_W=sum(YW)
		bys id: keep if _n==1
		merge 1:1 id using "`data'/model_predictions.dta"
		egen design=group(N_B N_W)
		sum design
		local max=r(max)
		foreach d of numlist 1/`max' {
			preserve
			keep if design==`d'
			count
			local N=r(N)
			local N_B=N_B[1]
			local N_W=N_W[1]
			local M=(`N_B'+1)*(`N_W'+1)-1
			matrix f=J(`M',1,.)
			matrix f_star=J(`M',1,.)
			matrix F=J(`M',`M',0)
			local r=1
			foreach c_w of numlist 0/`N_W' {
				foreach c_b of numlist 0/`N_B' {
					if ((`c_w'>0) | (`c_b'>0)) {
						sum fsim_`c_w'_`c_b'
						matrix f_star[`r',1]=r(mean)
						matrix T[`t',1]=r(mean)
						gen E=(C_W==`c_w')*(C_B==`c_b')
						sum E
						matrix f[`r',1]=r(mean)
						matrix F[`r',`r']=r(mean)
						matrix T[`t',2]=r(mean)
						matrix T[`t',3]=`N_W'
						matrix T[`t',4]=`N_B'
						drop E
						local ++r
						local ++t
					}
				}
			}
			matrix V=F - (f*f')
			matrix C_`d'=`N'*(f - f_star)'*inv(V)*(f - f_star)
			local df_`d'=`r'-1
			disp "`N_W', `N_B':"
			matrix list C_`d'
			disp `df_`d''
			local C=`C'+C_`d'[1,1]
			local df=`df'+`df_`d''
			restore
		}
		local p=1-chi2(`df',`C')
		disp "Chi-sq. `df': `C', p = `p'"
		
		
		****Graph results
		clear
		svmat T
		ren T1 f_star
		ren T2 f
		ren T3 N_W
		ren T4 N_B
		graph twoway scatter f f_star if N_W==2 & N_B==2 ///
					|| scatter f f_star if N_W==3 & N_B==1 ///
					|| scatter f f_star if N_W==1 & N_B==3 ///
					|| scatter f f_star if N_W==4 & N_B==0 ///
					|| scatter f f_star if N_W==0 & N_B==4 ///
					|| function y = x, range(f_star) lpattern(dash) lcolor(black) ///
					xtitle("Model-predicted frequency") ytitle("Empirical frequency") ylabel(, nogrid) ///
					xlabel(0(0.02)0.1) ///
					text(0.12 0.02 "{it:{&chi}}{sup:2}(30) = 32.10") ///
					text(0.11 0.02 "{it:p} = 0.362") scheme(s1color) ///
					legend(lab(1 "(2,2) design") lab(2 "(3,1) design") lab(3 "(1,3) design") lab(4 "(4,0) design") lab(5 "(0,4) design") order(1 2 3 4 5))
			
		
	}
	
	
	**************************************************
	**** UNCONSTRAINED MOMENTS (TABLES AI-AIII) ******
	**************************************************	
	if `moments'==1 {
	
		
			********BM moment estimates****************
			
				*Load data
				use "`data'/bm.dta", clear
				
				*Collapse data to job level
				gen Y_B=Y*B
				gen Y_W=Y*(1-B)
				bys id: egen C_b=sum(Y_B)
				bys id: egen C_w=sum(Y_W)
				bys id: keep if _n==1
					
				*Construct list of moments
				local f=""
				foreach l of numlist 0/2 {
					foreach k of numlist 0/2 {
						if (`l'>0) | (`k'>0) {
							cap gen f_`l'`k'=(C_w==`l')*(C_b==`k')
						}
					}
				}
				foreach w of numlist 0/2 {
					foreach b of numlist 0/2 {
							
						if (`w'>0) | (`b'>0) {
							cap gen f_`w'`b'=(C_w==`w')*(C_b==`b')
							local K=2-`w'
							local J=2-`b'
							local f_`w'`b'="0"
							foreach k of numlist 0/`K' {
								foreach j of numlist 0/`J' {
									local mw = 2-`k'
									local mb=2-`j'
									if (`mw'>0) | (`mb'>0) {
										local coef= comb(2,`w')*comb(2,`b')*((-1)^(4-(`w'+`b'+`k'+`j')))*comb(2-`w',`k')*comb(2-`b',`j')
										local f_`w'`b' "`f_`w'`b''+`coef'*{m_`mw'`mb'}"
									}
								}
							}		
						}
					}
				}

				*GMM estimates
				gmm ((f_22 - (`f_22'))) ///
					((f_21 - (`f_21'))) ///
					((f_12 - (`f_12'))) ///
					((f_20 - (`f_20'))) ///
					((f_02 - (`f_02'))) ///
					((f_11 - (`f_11'))) ///
					((f_10 - (`f_10'))) ///
					((f_01 - (`f_01'))) ///
					, winitial(identity)

				*Report centered moments
				foreach w of numlist 0/2 {
					foreach b of numlist 0/2 {
						if ((`w'+`b')>0) & ((`w'+`b')<=4) {
						
							*Calculate moment
							local m_`w'`b'="0"
							foreach j of numlist 0/`w' {
								foreach l of numlist 0/`b' {

										local coef=comb(`w',`j')*comb(`b',`l')*((-_b[m_10:_cons])^(`w'-`j'))*((-_b[m_01:_cons])^(`b'-`l'))
										if (`j'==0) & (`l'==0) {
											local m_`w'`b' "`m_`w'`b'' + `coef'"
										}
										if (`j'>0) | (`l'>0) {
											local m_`w'`b' "`m_`w'`b'' + `coef'*_b[m_`j'`l':_cons]"
										}		
									}
								}
								
								*Display estimate and SE
								disp "Moment m(`w',`b')"
								disp `m_`w'`b''
								lincom `m_`w'`b''
								
							}
						}
					}	
				
			
			********NPRS moment estimates****************

				*Load data
				use "`data'/nprs.dta", clear
			
				*Collapse to job level
				bys id: egen N_B=sum(B)
				gen N_W = N - N_B
				gen Y_B=Y*B
				gen Y_W=Y*(1-B)
				bys id: egen C_b=sum(Y_B)
				bys id: egen C_w=sum(Y_W)
				bys id: keep if _n==1
				gen D_22 = N_B==2 & N_W==2
				gen D_31 = N_B==1 & N_W==3
				gen D_13 = N_B==3 & N_W==1
				gen D_40 = N_B==0 & N_W==4
				gen D_04 = N_B==4 & N_W==0
				drop if D_40==1 | D_04==1

				*Construct moments
				local f=""
				foreach l of numlist 0/3 {
					foreach k of numlist 0/3 {
						if (`l'>0) | (`k'>0) {
							cap gen f_`l'`k'=(C_w==`l')*(C_b==`k')
						}
					}
				}
			
				*Design-specific estimates
	
					**(2,2)
					foreach w of numlist 0/2 {
						foreach b of numlist 0/2 {
						
							if (`w'>0) | (`b'>0) {
							
							cap gen f_`w'`b'=(C_w==`w')*(C_b==`b')
							local K=2-`w'
							local J=2-`b'
							local f_`w'`b'_D22="0"
							local f_`w'`b'_D22_sep="0"
							foreach k of numlist 0/`K' {
								foreach j of numlist 0/`J' {
									local mw = 2-`k'
									local mb=2-`j'
									if (`mw'>0) | (`mb'>0) {
										local coef= comb(2,`w')*comb(2,`b')*((-1)^(4-(`w'+`b'+`k'+`j')))*comb(2-`w',`k')*comb(2-`b',`j')
										local f_`w'`b'_D22 "`f_`w'`b'_D22'+`coef'*{m_`mw'`mb'}"
										local f_`w'`b'_D22_sep "`f_`w'`b'_D22_sep'+`coef'*{m_`mw'`mb'_D22}"
									}
								}
							}
							
							
							}
						}
					}
					
					**(3,1)
					foreach w of numlist 0/3 {
						foreach b of numlist 0/1 {
						
							if (`w'>0) | (`b'>0) {
							
							cap gen f_`w'`b'=(C_w==`w')*(C_b==`b')
							local K=3-`w'
							local J=1-`b'
							local f_`w'`b'_D31="0"
							local f_`w'`b'_D31_sep="0"
							foreach k of numlist 0/`K' {
								foreach j of numlist 0/`J' {
									local mw = 3-`k'
									local mb=1-`j'
									if (`mw'>0) | (`mb'>0) {
										local coef= comb(3,`w')*comb(1,`b')*((-1)^(4-(`w'+`b'+`k'+`j')))*comb(3-`w',`k')*comb(1-`b',`j')
										local f_`w'`b'_D31 "`f_`w'`b'_D31'+`coef'*{m_`mw'`mb'}"
										local f_`w'`b'_D31_sep "`f_`w'`b'_D31_sep'+`coef'*{m_`mw'`mb'_D31}"
									}
								}
							}
							
							
							}
						}
					}
					
					**(1,3)
					foreach w of numlist 0/1 {
						foreach b of numlist 0/3 {
						
							if (`w'>0) | (`b'>0) {
							
							cap gen f_`w'`b'=(C_w==`w')*(C_b==`b')
							local K=1-`w'
							local J=3-`b'
							local f_`w'`b'_D13="0"
							local f_`w'`b'_D13_sep="0"
							foreach k of numlist 0/`K' {
								foreach j of numlist 0/`J' {
									local mw = 1-`k'
									local mb=3-`j'
									if (`mw'>0) | (`mb'>0) {
										local coef= comb(1,`w')*comb(3,`b')*((-1)^(4-(`w'+`b'+`k'+`j')))*comb(1-`w',`k')*comb(3-`b',`j')
										local f_`w'`b'_D13 "`f_`w'`b'_D13'+`coef'*{m_`mw'`mb'}"
										local f_`w'`b'_D13_sep "`f_`w'`b'_D13_sep'+`coef'*{m_`mw'`mb'_D13}"
									}
								}
							}
							
							
							}
						}
					}
					
					
				
					***Fit model
					gmm (D_22*(f_22 - (`f_22_D22_sep'))) /// 2x2 design
					(D_22*(f_21 - (`f_21_D22_sep'))) ///
					(D_22*(f_12 - (`f_12_D22_sep'))) ///
					(D_22*(f_20 - (`f_20_D22_sep'))) ///
					(D_22*(f_02 - (`f_02_D22_sep'))) ///
					(D_22*(f_11 - (`f_11_D22_sep'))) ///
					(D_22*(f_10 - (`f_10_D22_sep'))) ///
					(D_22*(f_01 - (`f_01_D22_sep'))) ///
					(D_31*(f_31 - (`f_31_D31_sep'))) /// 3x1 design 
					(D_31*(f_30 - (`f_30_D31_sep'))) ///
					 (D_31*(f_21 - (`f_21_D31_sep'))) ///
					 (D_31*(f_20 - (`f_20_D31_sep'))) ///
					 (D_31*(f_11 - (`f_11_D31_sep'))) ///
					 (D_31*(f_10 - (`f_10_D31_sep'))) ///
					 (D_31*(f_01 - (`f_01_D31_sep'))) ///
					 (D_13*(f_13 - (`f_13_D13_sep'))) /// 1x3 design 
					 (D_13*(f_03 - (`f_03_D13_sep'))) ///
					 (D_13*(f_12 - (`f_12_D13_sep'))) ///
					 (D_13*(f_02 - (`f_02_D13_sep'))) ///
					 (D_13*(f_11 - (`f_11_D13_sep'))) ///
					 (D_13*(f_01 - (`f_01_D13_sep'))) ///
					 (D_13*(f_10 - (`f_10_D13_sep'))) /// 
					, winitial(identity) onestep
				
				   **Compute expressions for centered moments
				   foreach w of numlist 0/4 {
						foreach b of numlist 0/4 {
							if ((`w'+`b')>0) & ((`w'+`b')<=4) {
							
								foreach d in 22 31 13 {
									local m_`w'`b'_`d'="0"
								}
								
								foreach j of numlist 0/`w' {
									foreach l of numlist 0/`b' {
									
									
										***2x2 design
										if (`w'<=2) & (`b'<=2) {
											local coef=comb(`w',`j')*comb(`b',`l')*((-_b[m_10_D22:_cons])^(`w'-`j'))*((-_b[m_01_D22:_cons])^(`b'-`l'))
											if (`j'==0) & (`l'==0) {
												local m_`w'`b'_22 "`m_`w'`b'_22' + `coef'"
											}
											if (`j'>0) | (`l'>0) {
												local m_`w'`b'_22 "`m_`w'`b'_22' + `coef'*_b[m_`j'`l'_D22:_cons]"
											}
										}

										***3x1 design
										if (`w'<=3) & (`b'<=1) {
											local coef=comb(`w',`j')*comb(`b',`l')*((-_b[m_10_D31:_cons])^(`w'-`j'))*((-_b[m_01_D31:_cons])^(`b'-`l'))
											if (`j'==0) & (`l'==0) {
												local m_`w'`b'_31 "`m_`w'`b'_31' + `coef'"
											}
											if (`j'>0) | (`l'>0) {
												local m_`w'`b'_31 "`m_`w'`b'_31' + `coef'*_b[m_`j'`l'_D31:_cons]"
											}	
										}
										
										***1x3 design
										if (`w'<=1) & (`b'<=3) {
											local coef=comb(`w',`j')*comb(`b',`l')*((-_b[m_10_D13:_cons])^(`w'-`j'))*((-_b[m_01_D13:_cons])^(`b'-`l'))
											if (`j'==0) & (`l'==0) {
												local m_`w'`b'_13 "`m_`w'`b'_13' + `coef'"
											}
											if (`j'>0) | (`l'>0) {
												local m_`w'`b'_13 "`m_`w'`b'_13' + `coef'*_b[m_`j'`l'_D13:_cons]"
											}	
										}
										
										
									}
								}
									
								
								foreach d in 22 31 13  {
									disp "Moment m(`w',`b'), design (`d'):"
									disp `m_`w'`b'_`d''
									lincom `m_`w'`b'_`d''
								}
							
							}
						}
					}					
				   
			   
				   **Test restrictions
				   test (_b[m_10_D22:_cons]=_b[m_10_D31:_cons]) (_b[m_10_D22:_cons]=_b[m_10_D13:_cons])
				   test (_b[m_01_D22:_cons]=_b[m_01_D13:_cons]) (_b[m_01_D22:_cons]=_b[m_01_D31:_cons])
				   test (`m_20_22'=`m_20_31') 
				   test (`m_02_22'=`m_02_13') 
				   test (`m_11_22'=`m_11_13') (`m_11_22'=`m_11_31')
				   test (`m_21_22'=`m_21_31')
				   test (`m_12_22'=`m_12_13')
			
			
			********AGCV moment estimates****************
			
				*Load data
				use "`data'/agcv.dta", clear
				
				*Collapse to job level
				gen Y_B=Y*B
				gen Y_W=Y*(1-B)
				bys id: egen C_b=sum(Y_B)
				bys id: egen C_w=sum(Y_W)
				bys id: keep if _n==1

				*Construct moments
				local f=""
				foreach l of numlist 0/4 {
					foreach k of numlist 0/4 {
						if (`l'>0) | (`k'>0) {
							cap gen f_`l'`k'=(C_w==`l')*(C_b==`k')
						}
					}
				}

				*Binomial coefficients for (4,4) design
				foreach w of numlist 0/4 {
					foreach b of numlist 0/4 {
					
						if (`w'>0) | (`b'>0) {
						
							cap gen f_`w'`b'=(C_w==`w')*(C_b==`b')
							local K=4-`w'
							local J=4-`b'
							local f_`w'`b'_D44="0"
							local f_`w'`b'_D44_sep="0"
							foreach k of numlist 0/`K' {
								foreach j of numlist 0/`J' {
									local mw = 4-`k'
									local mb=4-`j'
									if (`mw'>0) | (`mb'>0) {
										local coef= comb(4,`w')*comb(4,`b')*((-1)^(8-(`w'+`b'+`k'+`j')))*comb(4-`w',`k')*comb(4-`b',`j')
										local f_`w'`b'_D44 "`f_`w'`b'_D44'+`coef'*{m_`mw'`mb'}"
									}
								}
							}
						}
					}
				}
					
				

				*Fit model
				gmm (D_44*(f_44 -(`f_44_D44'))) /// 4x4 design
					(D_44*(f_43 -(`f_43_D44'))) ///
					(D_44*(f_42 -(`f_42_D44'))) ///
					(D_44*(f_41 -(`f_41_D44'))) ///
					(D_44*(f_40 -(`f_40_D44'))) ///
					(D_44*(f_34 -(`f_34_D44'))) ///
					(D_44*(f_33 -(`f_33_D44'))) ///
					(D_44*(f_32 -(`f_32_D44'))) ///
					(D_44*(f_31 -(`f_31_D44'))) ///
					(D_44*(f_30 -(`f_30_D44'))) ///
					(D_44*(f_24 -(`f_24_D44'))) ///
					(D_44*(f_23 -(`f_23_D44'))) ///
					(D_44*(f_22 -(`f_22_D44'))) ///
					(D_44*(f_21 -(`f_21_D44'))) ///
					(D_44*(f_20 -(`f_20_D44'))) ///
					(D_44*(f_14 -(`f_14_D44'))) ///
					(D_44*(f_13 -(`f_13_D44'))) ///
					(D_44*(f_12 -(`f_12_D44'))) ///
					(D_44*(f_11 -(`f_11_D44'))) ///
					(D_44*(f_10 -(`f_10_D44'))) ///
					(D_44*(f_04 -(`f_04_D44'))) ///
					(D_44*(f_03 -(`f_03_D44'))) ///
					(D_44*(f_02 -(`f_02_D44'))) ///
					(D_44*(f_01 -(`f_01_D44'))) ///
				, winitial(identity) onestep

				
				
			   *Compute expressions for centered moments
			   foreach w of numlist 0/4 {
					foreach b of numlist 0/4 {
						if ((`w'+`b')>0) & ((`w'+`b')<=8) {

							local m_`w'`b'="0"

								foreach j of numlist 0/`w' {
									foreach l of numlist 0/`b' {
									
									
										***4x4 design
										if (`w'<=4) & (`b'<=4) {
											local coef=comb(`w',`j')*comb(`b',`l')*((-_b[m_10:_cons])^(`w'-`j'))*((-_b[m_01:_cons])^(`b'-`l'))
											if (`j'==0) & (`l'==0) {
												local m_`w'`b' "`m_`w'`b'' + `coef'"
											}
											if (`j'>0) | (`l'>0) {
												local m_`w'`b' "`m_`w'`b'' + `coef'*_b[m_`j'`l':_cons]"
											}
										}

										
									}
								}

								disp "Moment m(`w',`b'):"
								disp `m_`w'`b''
								lincom `m_`w'`b''
								
						}
					}
				}	

	}
	
	
	****************************************************
	**** EXPORT PREDICTED FREQUENCIES FROM LOGIT MODEL *
	****************************************************
	if `logit_frequencies'==1 {
	
			/* This section of code uses the estimated logit parameters to export
			    predicted callback frequencies and logit posteriors for various 
				experimental designs, which will then be used in subsequent programs.
				The version below generates predictions for 4-application designs. To
				produce the file with 10 application designs, which is labeled
				"logit_frequencies_withX_5_5.csv", change A_w_min, A_w_max, A_b_min, 
				and A_b_max to 5, and change A_min, and A_max to 10, then run again.*/
	
	
			*****Basic setup
			cap set more off
			cap set trace off
			cap log close
			set matsize 10000
			set seed 10141985
			
			******Define logit parameters
			local alpha_0=-4.931
			local sigma_alpha=4.988
			local beta_0=4.046
			local tau_0=-1.586
			local tau_alpha=0
			
			*****Define design parameters
			local J=50000000
			local A_w_min=1
			local A_w_max=3
			local A_b_min=1
			local A_b_max=3
			local A_min=4
			local A_max=4
			
			******Set up results storage
			matrix T=J(10000,13,.)
			local r=1
			
			******Simulate p's
			set obs `J'
			gen alpha=`alpha_0'+`sigma_alpha'*invnormal(uniform())
			gen p_disc=1/(1+exp(-(`tau_0'+`tau_alpha'*alpha)))
			gen D=uniform()<=p_disc
			gen beta=-D*`beta_0'
			gen p_w_h=1/(1+exp(-(alpha+0.5)))
			gen p_w_l=1/(1+exp(-(alpha-0.5)))
			gen p_b_h=1/(1+exp(-(alpha+beta+0.5)))
			gen p_b_l=1/(1+exp(-(alpha+beta-0.5)))
			sum p_disc
			local P_D=r(mean)
			gen pos_h = p_b_h - 0.8*p_w_h
			replace pos_h=0 if pos_h<=0
			gen pos_l = p_b_l - 0.8*p_w_l
			replace pos_l=0 if pos_l<=0
			gen pos = pos_h + pos_l 
			gen neg_h=0.8*p_w_h - p_b_h
			replace neg_h=0 if neg_h<=0
			gen neg_l=0.8*p_w_l - p_b_l
			replace neg_l=0 if neg_l<=0
			gen neg=neg_h+neg_l
			gen ratio=(p_w_l+p_w_h)/(p_b_l+p_b_h)
			gen logit_ratio=exp(ratio)/(1+exp(ratio))

			******Loop over possible designs
			foreach A_w of numlist `A_w_min'/`A_w_max' {
				foreach A_b of numlist `A_b_min'/`A_b_max' {
					if ((`A_w'+`A_b')<=`A_max') & ((`A_w'+`A_b')>=`A_min') {
						
						*****Loop over possible X configurations
						foreach A_w_h of numlist 0/`A_w' {
							foreach A_b_h of numlist 0/`A_b' {
								local A_w_l = `A_w'-`A_w_h'
								local A_b_l = `A_b'-`A_b_h'
								
								******Simulate callback outcomes in this design
								foreach g in w_h w_l b_h b_l {
									qui gen C_`g'=0
									if `A_`g''>0 {
										qui replace C_`g'=rbinomial(`A_`g'',p_`g')
									}
								}

								
								*****Loop over possible callback outcomes 
								foreach C_w_h of numlist 0/`A_w_h' {
									foreach C_w_l of numlist 0/`A_w_l' {
										foreach C_b_h of numlist 0/`A_b_h' {
											foreach C_b_l of numlist 0/`A_b_l' {
											
												********Calculate frequency
												qui gen f=1
												 foreach t in w_h w_l b_h b_l {
													gen f_`t'=1
													if `A_`t''>0 {
														qui replace f_`t'=comb(`A_`t'',`C_`t'')*(p_`t'^`C_`t'')*((1-p_`t')^(`A_`t''-`C_`t''))
													}
													qui replace f=f*f_`t'
												 }
												 

												 *****Fill in results
												 matrix T[`r',1]=`A_w_h'
												 matrix T[`r',2]=`A_w_l'
												 matrix T[`r',3]=`A_b_h'
												 matrix T[`r',4]=`A_b_l'
												 matrix T[`r',5]=`C_w_h'
												 matrix T[`r',6]=`C_w_l'
												 matrix T[`r',7]=`C_b_h'
												 matrix T[`r',8]=`C_b_l'
												 qui sum f
												 local f=r(mean)
												 matrix T[`r',9]=`f'
												 qui sum f if D==1
												 matrix T[`r',10]=r(mean)*(`P_D'/`f')
												 qui sum pos if C_w_h==`C_w_h' & C_w_l==`C_w_l' & C_b_h==`C_b_h' & C_b_l==`C_b_l'
												 matrix T[`r',11]=r(mean)
												 qui sum neg if C_w_h==`C_w_h' & C_w_l==`C_w_l' & C_b_h==`C_b_h' & C_b_l==`C_b_l'
												 matrix T[`r',12]=r(mean)
												 qui sum logit_ratio if C_w_h==`C_w_h' & C_w_l==`C_w_l' & C_b_h==`C_b_h' & C_b_l==`C_b_l'
												 matrix T[`r',13]=r(mean)
												 qui drop f*
												 local ++r
												 disp `r'
											}
										}
									}
								}
								
								***Drop simulated callbacks
								qui drop C*
								
							}
						}
					}
				}
			}
			
			
			*******Save results
			clear
			svmat T
			ren T1 A_w_h
			ren T2 A_w_l
			ren T3 A_b_h
			ren T4 A_b_l
			ren T5 C_w_h
			ren T6 C_w_l
			ren T7 C_b_h
			ren T8 C_b_l
			ren T9 f
			ren T10 post
			ren T11 E_pos
			ren T12 E_neg
			ren T13 E_logit
			drop if f==.
			outsheet using "`data'/logit_frequencies_withX_2_2.csv", names comma replace
	
	}
		

	**************************************************
	**** MAKE GRIDS FOR MATLAB QP AND LP PROGRAMS ****
	**************************************************	
	if `grids'==1 {
	
		* a) rectangular grids
		foreach K of numlist 150 900 { 
			clear all
			set obs `K'
			gen i=_n 
			expand `K'
			bysort i:gen j=_n 
			gen p_w = (i-1)/`K'
			gen p_b = (j-1)/`K'
			drop i j
			compress
			export delimited using "`grids'/grid_rect_K`K'.csv", replace 
		}

		* b) quadratic grids -- includes discretized logit G (PMF)

		* Set parameters	
		clear all
		local tau_0=-1.586
		local tau_alpha=0
		local beta_0=4.046
		local mu_alpha=-4.931
		local sigma_alpha=4.988
		local J=10000000
			
		* Draw probabilities
		set obs `J'
		g alpha  =`mu_alpha'+`sigma_alpha'*invnormal(uniform())
		g u_type =`tau_0'+`tau_alpha'*alpha
		g p_type = 1/(1+exp(-u_type))
		g type   = p_type>=uniform()
		g beta   = `beta_0'*type
		g p_w	 = 1/(1+exp(-alpha))
		g p_b	 = 1/(1+exp(-(alpha-beta)))

		foreach K of numlist 50 100 150 200 300 400 600 900 { 
			preserve
			* Collapse to Quadratic Ls grid scheme
			g p_w_t = (`K'-1)*p_w + 1 
			g p_b_t = (`K'-1)*p_b + 1
			g i 	= . 
			g j 	= .
			replace i = round(p_w_t) if p_w_t <= p_b_t
			replace j = round(p_b_t) if p_b_t <= p_w_t
			replace i = round(sqrt((`K'-j)*(p_w_t-j))+j) if p_b_t < p_w_t
			replace j = round(sqrt((`K'-i)*(p_b_t-i))+i) if p_w_t < p_b_t
			gen 	G = 1 
			collapse (count) G, by(i j)
			replace G = G/`J'
			tempfile tmp
			save "`tmp'" 
			clear
			set obs `K'
			gen i=_n //make rows
			expand `K'
			bysort i:gen j=_n //make cols
			gen p_w=((max(0,i-j))^2/(`K'+1-j)+min(i,j)-1)/(`K') 
			gen p_b=((max(0,j-i))^2/(`K'+1-i)+min(i,j)-1)/(`K') 
			merge 1:1 i j using "`tmp'" , nogen 
			replace G = 0 if mi(G)
			sort j i 
			drop j i
			outsheet using "`grids'/logit_G_quadL_`K'.csv", comma replace 
			restore
		} 


		* c) 4D quadratc grid
		clear
		set matsize 10000
		set seed 10141985

		* Define logit parameters
		local alpha_0=-4.931
		local sigma_alpha=4.988
		local beta_0=4.046
		local tau_0=-1.586
		local tau_alpha=0

		* efine simulation parameters
		local J = 50000000
		local K = 55

		******Simulate p's
		set obs `J'
		g alpha=`alpha_0'+`sigma_alpha'*invnormal(uniform())
		g p_disc=1/(1+exp(-(`tau_0'+`tau_alpha'*alpha)))
		g D=uniform()<=p_disc
		g beta=-D*`beta_0'
		g p_w_h=1/(1+exp(-(alpha+0.5)))
		g p_w_l=1/(1+exp(-(alpha-0.5)))
		g p_b_h=1/(1+exp(-(alpha+beta+0.5)))
		g p_b_l=1/(1+exp(-(alpha+beta-0.5)))

		****Collapse to 2D Quadratic L's grid scheme
		g p_w_h_t 	= (`K'-1)*p_w_h + 1 
		g p_b_h_t 	= (`K'-1)*p_b_h + 1
		g p_w_l_t 	= (`K'-1)*p_w_l + 1 
		g p_b_l_t 	= (`K'-1)*p_b_l + 1
		g i_h 		= round(p_w_h_t) if p_w_h_t <= p_b_h_t
		g j_h 		= round(p_b_h_t) if p_b_h_t <= p_w_h_t
		g i_l 		= round(p_w_l_t) if p_w_l_t <= p_b_l_t
		g j_l 		= round(p_b_l_t) if p_b_l_t <= p_w_l_t
		replace i_h = round(sqrt((`K'-j_h)*(p_w_h_t-j_h))+j_h) if p_b_h_t < p_w_h_t
		replace j_h = round(sqrt((`K'-i_h)*(p_b_h_t-i_h))+i_h) if p_w_h_t < p_b_h_t
		replace i_l = round(sqrt((`K'-j_l)*(p_w_l_t-j_l))+j_l) if p_b_l_t < p_w_l_t
		replace j_l = round(sqrt((`K'-i_l)*(p_b_l_t-i_l))+i_l) if p_w_l_t < p_b_l_t
		gen G 		= 1 
		collapse (count) G, by(i_h j_h i_l j_l)
		replace G 	= G/`J'
		tempfile tmp
		save "`tmp'" 
		* Create 4D full grid
		clear
		set obs `K'
							gen i_h=_n 
		expand `K'
		bysort i_h:			gen j_h=_n 
		expand `K'
		bysort i_h j_h:		gen i_l=_n 
		expand `K'
		bysort i_h j_h i_l:	gen j_l=_n 
		gen p_w_h=((max(0,i_h-j_h))^2/(`K'+1-j_h)+min(i_h,j_h)-1)/(`K') 
		gen p_b_h=((max(0,j_h-i_h))^2/(`K'+1-i_h)+min(i_h,j_h)-1)/(`K') 
		gen p_w_l=((max(0,i_l-j_l))^2/(`K'+1-j_l)+min(i_l,j_l)-1)/(`K') 
		gen p_b_l=((max(0,j_l-i_l))^2/(`K'+1-i_l)+min(i_l,j_l)-1)/(`K')
		drop if p_w_l > p_w_h 
		drop if p_b_l > p_b_h 
		drop if p_b_h > p_w_h 
		drop if p_b_l > p_w_l 
		merge 1:1 i_h j_h i_l j_l using "`tmp'" , nogen 
		replace G = 0 if mi(G)
		drop if p_w_l > p_w_h 
		drop if p_b_l > p_b_h 
		drop if p_b_h > p_w_h 
		drop if p_b_l > p_w_l 
		drop i_h j_h i_l j_l 
		export delimited using "`grids'/logit_grid_QLmono_withX_K`K'csv", replace 
		
	}
	
