function [prob_disc ep_w ep_b posterior_ratio bayes_factor] = bayes_normal(sim, prior_params, history)
	% Function computes posterior moments based on normal distribution of alpha_j and discrete types for beta_j
	% sim -- dimension of grid for numerical evaluation of posterior
	% beta -- Fixed estimate of beta parameter for types model 
	% beta_share -- Prior for share of beta != 0 firms
	% prior_params -- parameters of alpha_j normal distribution for type1 and type2, and bet share parameters, respectively
	% history -- vector of previous trials (possibly empty) as [N_white rate_white N_black rate_black]

	%%% 0) Define a few variables based on the inputs
	mu1 = prior_params(1); 
	sigma1 = prior_params(2); 		% Note this should be the SD -- not variance
	mu2 = prior_params(3); 
	sigma2 = prior_params(4); 		% Note this should be the SD -- not variance, (can also be the same as type1)
	beta = prior_params(5);
	beta_share_d0 = prior_params(6);
	beta_share_da = prior_params(7);
	
	% This is the unconditoinal Pr(D=1) = E[logit(d_0 + d_a a_j)]
	beta_share = prior_params(8); 

	% These are the covariate bins 
	xbs = prior_params(9:end);
	num_xbs = size(xbs,2);

	%%% 1) First set up grid and calculate posterior density for grid of alpha_js, assuming 
	% You can just as easily define the alphas to be the quad points and weights?
	i   = 1:sim-1;
	a   = sqrt(i/2);
	CM  = diag(a,1) + diag(a,-1);
	[V L]   = eig(CM);
	[x ind] = sort(diag(L));
	V       = V(:,ind)';
	w       = sqrt(pi) * V(:,1).^2;

	alphas = 2^0.5*sigma1*x' + mu1;
	weights = w' / pi^0.5;

	% Define matrix of pws (sims by dim(types))
	probs_w = zeros(sim,num_xbs);
	for c = 1:num_xbs
		probs_w(:,c) = (1 + exp(-alphas - xbs(c))).^-1;
	end

	% Now also define a matrix of pbs
	probs_b = zeros(sim,num_xbs);
	for c = 1:num_xbs
		probs_b(:,c) = (1 + exp(-alphas - xbs(c) + beta)).^-1;
	end

	% And likelihood of youre a discriminator is also a strict function of alphas
	probs_D = (1 + exp(-beta_share_d0 - alphas*beta_share_da)).^-1;

	% Define posterior for type 1 -> not racist
	p_h_mu1 = prod(probs_w.^(history(1:4:4*num_xbs).*history(2:4:4*num_xbs)).*(1-probs_w).^(history(1:4:4*num_xbs).*(1-history(2:4:4*num_xbs))) ...
				.* probs_w.^(history(3:4:4*num_xbs).*history(4:4:4*num_xbs)).*(1-probs_w).^(history(3:4:4*num_xbs).*(1-history(4:4:4*num_xbs))),2);
	p_h_mu1 =  (p_h_mu1 .* weights' .* (1-probs_D'))';

	% Now define the posterior likelihood assuming type 2, which implies p_w, p_b joint as above
	p_h_mu2 = prod(probs_w.^(history(1:4:4*num_xbs).*history(2:4:4*num_xbs)).*(1-probs_w).^(history(1:4:4*num_xbs).*(1-history(2:4:4*num_xbs))) ...
				.* probs_b.^(history(3:4:4*num_xbs).*history(4:4:4*num_xbs)).*(1-probs_b).^(history(3:4:4*num_xbs).*(1-history(4:4:4*num_xbs))),2);
	p_h_mu2 =  (p_h_mu2 .* weights' .* probs_D')';
	
	% Joint probability needs to integrate / sum to 1, so normalize by sum of total 
	tot = sum(p_h_mu1) + sum(p_h_mu2);
	p_h_mu1 = p_h_mu1 / tot;
	p_h_mu2 = p_h_mu2 / tot;

	%%% 2) Now save likelihood that firm is discriminator
	prob_disc = sum(p_h_mu2);
	posterior_ratio = prob_disc/(1-prob_disc);
	bayes_factor = posterior_ratio * (1-beta_share) / beta_share;

	%%% 3) Now save expected p_w and p_b from posterior (note that these are now vectors of dim(num covar types))
	ep_w = sum(probs_w.*p_h_mu1' + probs_w.*p_h_mu2');	
	ep_b = sum(probs_w.*p_h_mu1' + probs_b.*p_h_mu2');

end
