function ans = portfolio_test(num_initial_pairs, max_depth, pairs, xbs, randomize)
tic
%%% Test out the policy here
% Set seed
rng(6583939)

try	
	num_initial_pairs = str2num(num_initial_pairs)
	max_depth = str2num(max_depth)
	pairs = str2num(pairs)
	xbs = str2num(xbs)
	touse = str2num(getenv('SLURM_ARRAY_TASK_ID'))	
	randomize = str2num(randomize)
	if max_depth > 30
		warning('max depth beyond possible draws limit')
	end
catch
	warning('failed to find max depth and initial pairs')
	num_initial_pairs = 3
	max_depth = 6
	pairs = 0
	xbs = 2
	randomize = 1
	touse = 200
end

%%% 1) Construct a sample of firms using prior DGP
num_firms = 500000;
num_sims = 100;

% Initialize prior parameters
prior_params = [-4.931 4.988 0 4.983 4.046 -1.586 0 0 0]	% these are the results from Aug 1. with covariates...
%prior_params = [-4.670 4.716 0 4.716 3.498 -1.518 0 0 0]
if xbs == 2
	prior_params = [-4.931 4.988 0 4.983 4.046 -1.586 0 0 -.50 .50]
	%prior_params = [-4.670 4.716 0 4.716 3.498 -1.518 0 0 -.51 .51]
end

% Declare number of covariate bins
cov_params = prior_params(9:end)
cov_bins = size(cov_params,2)

% Calculate unconditional prior share D  
prd = @(a) (1 + exp(-(prior_params(6) + prior_params(7)*a))).^-1 .* ...
			normpdf(a,prior_params(1),prior_params(2));
prior_params(8) = integral(prd,-Inf,Inf);

% Set firm types and probabilities
firm_alphas = normrnd(prior_params(1),prior_params(2),num_firms,1);
firm_D_probs = (1 + exp(-(prior_params(6) + prior_params(7)*firm_alphas))).^-1;
firm_types = binornd(1, firm_D_probs, num_firms,1);

p_w = zeros(num_firms,cov_bins);
p_b = zeros(num_firms,cov_bins);
for k = 1:cov_bins
	p_w(:,k) = arrayfun(@(x) (1+exp(-x - cov_params(k)))^-1, firm_alphas);
	p_b(:,k) = (1-firm_types) .* p_w(:,k) + firm_types .* arrayfun(@(x) (1+exp(-x- cov_params(k)))^-1, firm_alphas - prior_params(5));
end

% Generate draws of all possible squences for each firm and race / type combination
possible_draws = zeros(num_firms,30,2*cov_bins);
for k = 1:cov_bins
	possible_draws(:,:,1+2*(k-1)) = binornd(1,repmat(p_w(:,k),1,30));
	possible_draws(:,:,2+2*(k-1)) = binornd(1,repmat(p_b(:,k),1,30));
end 
possible_draws = possible_draws(:,1:max_depth,:); 


%%% 2) Generate initial histories based on firm probabilities
histories = zeros(num_firms,4*cov_bins);

% If randomize = 0, you get num_initial_pairs for each covariate type
if randomize == 0
	for k = 1:cov_bins
		histories(:,1+4*(k-1)) = num_initial_pairs;
		histories(:,3+4*(k-1)) = num_initial_pairs;
		
		% Correct for chattering in application draws
		histories(:,2+4*(k-1)) = sum(possible_draws(:,1:num_initial_pairs,1+2*(k-1)),2) / num_initial_pairs;
		histories(:,4+4*(k-1)) = sum(possible_draws(:,1:num_initial_pairs,2+2*(k-1)),2) / num_initial_pairs;
	end

% Otherwise we have to distribute initial pairs among types randomly
else
	choices = zeros(num_firms,cov_bins);
	choices(:,1) = randi([0 num_initial_pairs],num_firms,1);
	for k = 2:cov_bins
		for j = 1:num_firms
			choices(j,k) = randi([0 (num_initial_pairs-sum(choices(j,1:k-1)))],1,1);
		end
	end
	choices(:,end) = num_initial_pairs - sum(choices(:,1:cov_bins-1),2);
	
	for k = 1:cov_bins
		histories(:,1+4*(k-1)) = choices(:,k);
		histories(:,3+4*(k-1)) = choices(:,k);

		for p = 1:num_initial_pairs
			% Correct for chattering in application draws
			histories(choices(:,k) == p,2+4*(k-1)) = sum(possible_draws(choices(:,k) == p,1:p,1+2*(k-1)),2) / p;
			histories(choices(:,k) == p,4+4*(k-1)) = sum(possible_draws(choices(:,k) == p,1:p,2+2*(k-1)),2) / p;
		end
	end
end

% Replace NaNs with zeros if needed
histories(isnan(histories)) = 0;

orig_histories = histories;

%%% 3) Simulate auditors problem, using cost in top value
% Reset histories
histories = orig_histories;

uni = unique(histories,'rows');
indicators = zeros(num_firms,size(uni,1));
for type = 1:size(uni,1)
	indicators(:,type) = sum(abs(histories - uni(type,:)),2) == 0;
end

uniresults = zeros(size(uni,1),1);
for row = 1:size(uni,1)
	p = bayes_normal(num_sims, prior_params, uni(row,:));
	uniresults(row) = p;
end

initial_ps = indicators*uniresults;

% Set up grid of parameters to work over
min_ben = 100;
max_ben = 2000;
min_cost = 100;
max_cost = 2000;
incr = 100;
min_ac = 0.5;
max_ac = 0.5;
ac_incr = .1;
num_combs = ((max_ben - min_ben)/incr + 1) * ((max_cost - min_cost)/incr + 1) * ((max_ac - min_ac)/ac_incr + 1);
possible_params = zeros(num_combs,3);
i = 1;
for b = min_ben:incr:max_ben
	for c = min_cost:incr:max_cost
		for ac = min_ac:ac_incr:max_ac
			possible_params(i,:) = [b c ac];
			i = i+1;
		end
	end
end

params = [possible_params(touse,:) 1];		% cost here is 0.5 because sending 1/2 an pair
value_dict = nan(0,4*cov_bins + 1 + 2);
	% this thing stores history, recursion, choice, and max
	% so total dim = 4*cov_bins + 1 + 2

% Compute all possible histories if needed for optimal portfoilio
if num_initial_pairs*2 < max_depth
	sofar = cell(1,max_depth+1)
	sofar{1} = zeros(1,4*cov_bins);
	for inc = 2:max_depth+1
		toinc = sofar{1,inc-1};
		toadd = nan(0,4*cov_bins);
		for r = 1:size(toinc,1)
			for c = 1:2*cov_bins
				newrow = toinc(r,:);
				newrow(1,2*c-1) = newrow(1,2*c-1) + 1;
				toadd = [toadd ; newrow];
				newrow(1,2*c) = newrow(1,2*c) + 1;
				toadd = [toadd ; newrow];
			end
		end
		toadd = unique(toadd,'rows');
		sofar{1,inc} = toadd;
	end

	allhist = unique(cat(1,sofar{:}),'rows');
	for c = 1:2*cov_bins
		allhist(:,2*c) = allhist(:,2*c) ./ allhist(:,2*c-1);
	end
	allhist(isnan(allhist)) = 0;
	allhist = allhist(sum(allhist(:,1:2:4*cov_bins),2) == max_depth,:);
	size(allhist)

	% Compute likelihood and payoffs for each possible history
	xbs = prior_params(9:end);
	num_xbs = size(xbs,2);

	% Compute probability of every history under prior
	probs = zeros(size(allhist,1),1);
	for row = 1:size(allhist,1)
		probs(row) = prob_history(num_sims,prior_params,allhist(row,:));
	end	

	% Compute optimal choices for every history
	results = zeros(size(allhist,1),2);
	for row = 1:size(allhist,1)
		[t c] = policy_singles(params, allhist(row,:), prior_params, sum(allhist(row,1:2:4*cov_bins)), max_depth, value_dict);
		results(row,:) = [t c];
	end
	tmp2 = [allhist results probs];
	tmp2(tmp2(:,1) == 7,:)

	% Put together and solve for best history
	uni = unique(allhist(:,1:2:4*cov_bins),'rows');
	indicators = zeros(size(allhist,1),size(uni,1));
	for type = 1:size(uni,1)
		indicators(:,type) = sum(abs(allhist(:,1:2:4*cov_bins) - uni(type,:)),2) == 0;
	end

	evals = indicators'*(probs.*results(:,1));
	[best val] = max(evals)
	best_portfolio = uni(val,:)

	%%% 4) Now deploy best portfolio on all the firms'
	for k = 1:size(best_portfolio,2)
		histories(:,2*k-1) = best_portfolio(k);
		histories(:,2*k) = sum(possible_draws(:,1:best_portfolio(k),k),2);
	end		
	histories(:,2:2:4*cov_bins+1) = histories(:,2:2:4*cov_bins+1)./ histories(:,1:2:4*cov_bins);
	histories(isnan(histories)) = 0;
else
	best_portfolio = []
end

% Add the final result in each case
uni = unique(histories,'rows');
indicators = zeros(num_firms,size(uni,1));
for type = 1:size(uni,1)
	indicators(:,type) = sum(abs(histories - uni(type,:)),2) == 0;
end

uniresults = zeros(size(uni,1),2);
for row = 1:size(uniresults,1)
	[t c vd] = policy_singles(params, uni(row,:), prior_params, sum(uni(row,1:2:4*cov_bins)), max_depth, value_dict);
	uniresults(row,:) = [t c];
end

results = indicators*uniresults;

% Get total payout
apps_sent = sum(histories(:,1:2:cov_bins*4),2);
costs = params(3)*apps_sent;
benefits = zeros(num_firms,1) + (results(:,2) == 2*cov_bins+1).*firm_types*params(1) - (results(:,2) == 2*cov_bins+1).*(1-firm_types)*params(2);
costben = benefits - costs;

% Get a few statistics on accusations
tot_payout = sum(costben);

% Collect share parameters required and then add to overall results
share_discrim_accused = mean(results(firm_types == 1,2) == 2*cov_bins+1);
share_non_discrim_acquit = mean(results(firm_types == 0,2) ~= 2*cov_bins+1);

tot_discrim_accused = sum(results(firm_types == 1,2) == 2*cov_bins+1);
tot_non_discrim_acquit = sum(results(firm_types == 0,2) ~= 2*cov_bins+1);

apps_sent = sum(sum(histories(:,1:2:cov_bins*4))) / num_firms;
final_results = [params apps_sent tot_payout share_discrim_accused share_non_discrim_acquit tot_discrim_accused tot_non_discrim_acquit]

% Save to spreadsheet
outname = sprintf('results/stupid_par/portfolio_curve_500k_nochat_%scovtypes_%sipairs_%sdepth_%spairs_%s_%s_%s_%s.csv', num2str(cov_bins), num2str(num_initial_pairs), num2str(max_depth), num2str(pairs), num2str(params(1)), num2str(params(2)), num2str(params(3)), num2str(params(4)));
fid = fopen(outname,'w');
names = {'gamma' 'kappa' 'c' 'delta' 'apps_sent' 'tot_payout' 'share_d_accused' 'share_nond_acquit' 'tot_d_accused' 'tot_non_d_acquit'};
fprintf(fid, '%s, %s, %s, %s, %s, %s, %s, %s, %s, %s\n', names{1,:});
fclose(fid);
dlmwrite(outname, final_results, 'delimiter', ',', '-append');
dlmwrite(outname, [best_portfolio], 'delimiter', ',', '-append');
toc
end


